# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.common import tag
from openstack import exceptions
from openstack import resource
from openstack import utils


class Trunk(resource.Resource, tag.TagMixin):
    resource_key = 'trunk'
    resources_key = 'trunks'
    base_path = '/trunks'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'name', 'description', 'port_id', 'status', 'sub_ports',
        'project_id',
        is_admin_state_up='admin_state_up',
        **tag.TagMixin._tag_query_parameters
    )

    # Properties
    #: Trunk name.
    name = resource.Body('name')
    #: The ID of the project who owns the trunk. Only administrative
    #: users can specify a project ID other than their own.
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)
    #: The trunk description.
    description = resource.Body('description')
    #: The administrative state of the port, which is up ``True`` or
    #: down ``False``. *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: The ID of the trunk's parent port
    port_id = resource.Body('port_id')
    #: The status for the trunk. Possible values are ACTIVE, DOWN, BUILD,
    #: DEGRADED, and ERROR.
    status = resource.Body('status')
    #: A list of ports associated with the trunk.
    sub_ports = resource.Body('sub_ports', type=list)

    def add_subports(self, session, subports):
        url = utils.urljoin('/trunks', self.id, 'add_subports')
        resp = session.put(url, json={'sub_ports': subports})
        exceptions.raise_from_response(resp)
        self._body.attributes.update(resp.json())
        return self

    def delete_subports(self, session, subports):
        url = utils.urljoin('/trunks', self.id, 'remove_subports')
        resp = session.put(url, json={'sub_ports': subports})
        exceptions.raise_from_response(resp)
        self._body.attributes.update(resp.json())
        return self

    def get_subports(self, session):
        url = utils.urljoin('/trunks', self.id, 'get_subports')
        resp = session.get(url)
        exceptions.raise_from_response(resp)
        self._body.attributes.update(resp.json())
        return resp.json()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Keypair(resource.Resource):
    resource_key = 'keypair'
    resources_key = 'keypairs'
    base_path = '/os-keypairs'

    _query_mapping = resource.QueryParameters(
        'user_id')

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _max_microversion = '2.10'

    # Properties
    #: The date and time when the resource was created.
    created_at = resource.Body('created_at')
    #: A boolean indicates whether this keypair is deleted or not.
    is_deleted = resource.Body('deleted', type=bool)
    #: The short fingerprint associated with the ``public_key`` for
    #: this keypair.
    fingerprint = resource.Body('fingerprint')
    # NOTE: There is in fact an 'id' field. However, it's not useful
    # because all operations use the 'name' as an identifier.
    # Additionally, the 'id' field only appears *after* creation,
    # so suddenly you have an 'id' field filled in after the fact,
    # and it just gets in the way. We need to cover this up by listing
    # name as alternate_id and listing id as coming from name.
    #: The id identifying the keypair
    id = resource.Body('name')
    #: A name identifying the keypair
    name = resource.Body('name', alternate_id=True)
    #: The private key for the keypair
    private_key = resource.Body('private_key')
    #: The SSH public key that is paired with the server.
    public_key = resource.Body('public_key')
    #: The type of the keypair.
    type = resource.Body('type', default='ssh')
    #: The user_id for a keypair.
    user_id = resource.Body('user_id')

    def _consume_attrs(self, mapping, attrs):
        # TODO(mordred) This should not be required. However, without doing
        # it **SOMETIMES** keypair picks up id and not name. This is a hammer.
        if 'id' in attrs:
            attrs.setdefault('name', attrs.pop('id'))
        return super(Keypair, self)._consume_attrs(mapping, attrs)

    @classmethod
    def existing(cls, connection=None, **kwargs):
        """Create an instance of an existing remote resource.

        When creating the instance set the ``_synchronized`` parameter
        of :class:`Resource` to ``True`` to indicate that it represents the
        state of an existing server-side resource. As such, all attributes
        passed in ``**kwargs`` are considered "clean", such that an immediate
        :meth:`update` call would not generate a body of attributes to be
        modified on the server.

        :param dict kwargs: Each of the named arguments will be set as
                            attributes on the resulting Resource object.
        """
        # Listing KPs return list with resource_key structure. Instead of
        # overriding whole list just try to create object smart.
        if cls.resource_key in kwargs:
            args = kwargs.pop(cls.resource_key)
            kwargs.update(**args)
        return cls(_synchronized=True, connection=connection, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional import base


class TestQS(base.BaseFunctionalTest):

    def test_qs(self):
        sot = self.conn.compute.get_quota_set(
            self.conn.current_project_id
        )
        self.assertIsNotNone(sot.key_pairs)

    def test_qs_user(self):
        sot = self.conn.compute.get_quota_set(
            self.conn.current_project_id,
            user_id=self.conn.session.auth.get_user_id(self.conn.compute)
        )
        self.assertIsNotNone(sot.key_pairs)

    def test_update(self):
        sot = self.conn.compute.get_quota_set(
            self.conn.current_project_id
        )
        self.conn.compute.update_quota_set(
            sot,
            query={
                'user_id': self.conn.session.auth.get_user_id(
                    self.conn.compute)
            },
            key_pairs=100
        )

    def test_revert(self):
        self.conn.compute.revert_quota_set(
            self.conn.current_project_id,
            user_id=self.conn.session.auth.get_user_id(self.conn.compute)
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.compute.v2 import flavor
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'links': '2',
    'name': '3',
    'description': 'Testing flavor',
    'disk': 4,
    'os-flavor-access:is_public': True,
    'ram': 6,
    'vcpus': 7,
    'swap': 8,
    'OS-FLV-EXT-DATA:ephemeral': 9,
    'OS-FLV-DISABLED:disabled': False,
    'rxtx_factor': 11.0
}
DEFAULTS_EXAMPLE = {
    'links': '2',
    'original_name': IDENTIFIER,
    'description': 'Testing flavor',
}


class TestFlavor(base.TestCase):

    def setUp(self):
        super(TestFlavor, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = 1
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = flavor.Flavor()
        self.assertEqual('flavor', sot.resource_key)
        self.assertEqual('flavors', sot.resources_key)
        self.assertEqual('/flavors', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_commit)

        self.assertDictEqual({"sort_key": "sort_key",
                              "sort_dir": "sort_dir",
                              "min_disk": "minDisk",
                              "min_ram": "minRam",
                              "limit": "limit",
                              "marker": "marker",
                              "is_public": "is_public"},
                             sot._query_mapping._mapping)

    def test_make_basic(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
        self.assertEqual(BASIC_EXAMPLE['disk'], sot.disk)
        self.assertEqual(BASIC_EXAMPLE['os-flavor-access:is_public'],
                         sot.is_public)
        self.assertEqual(BASIC_EXAMPLE['ram'], sot.ram)
        self.assertEqual(BASIC_EXAMPLE['vcpus'], sot.vcpus)
        self.assertEqual(BASIC_EXAMPLE['swap'], sot.swap)
        self.assertEqual(BASIC_EXAMPLE['OS-FLV-EXT-DATA:ephemeral'],
                         sot.ephemeral)
        self.assertEqual(BASIC_EXAMPLE['OS-FLV-DISABLED:disabled'],
                         sot.is_disabled)
        self.assertEqual(BASIC_EXAMPLE['rxtx_factor'], sot.rxtx_factor)

    def test_make_defaults(self):
        sot = flavor.Flavor(**DEFAULTS_EXAMPLE)
        self.assertEqual(DEFAULTS_EXAMPLE['original_name'], sot.name)
        self.assertEqual(0, sot.disk)
        self.assertEqual(True, sot.is_public)
        self.assertEqual(0, sot.ram)
        self.assertEqual(0, sot.vcpus)
        self.assertEqual(0, sot.swap)
        self.assertEqual(0, sot.ephemeral)
        self.assertEqual(IDENTIFIER, sot.id)

    def test_flavor_id(self):
        id = 'fake_id'
        sot = flavor.Flavor(id=id)
        self.assertEqual(sot.id, id)
        sot = flavor.Flavor(name=id)
        self.assertEqual(sot.id, id)
        self.assertEqual(sot.name, id)
        sot = flavor.Flavor(original_name=id)
        self.assertEqual(sot.id, id)
        self.assertEqual(sot.original_name, id)

    def test_add_tenant_access(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        resp = mock.Mock()
        resp.body = None
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.post = mock.Mock(return_value=resp)

        sot.add_tenant_access(self.sess, 'fake_tenant')

        self.sess.post.assert_called_with(
            'flavors/IDENTIFIER/action',
            json={
                'addTenantAccess': {
                    'tenant': 'fake_tenant'}},
            headers={'Accept': ''}
        )

    def test_remove_tenant_access(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        resp = mock.Mock()
        resp.body = None
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.post = mock.Mock(return_value=resp)

        sot.remove_tenant_access(self.sess, 'fake_tenant')

        self.sess.post.assert_called_with(
            'flavors/IDENTIFIER/action',
            json={
                'removeTenantAccess': {
                    'tenant': 'fake_tenant'}},
            headers={'Accept': ''}
        )

    def test_get_flavor_access(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        resp = mock.Mock()
        resp.body = {'flavor_access': [
            {'flavor_id': 'fake_flavor',
             'tenant_id': 'fake_tenant'}
        ]}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.get = mock.Mock(return_value=resp)

        rsp = sot.get_access(self.sess)

        self.sess.get.assert_called_with(
            'flavors/IDENTIFIER/os-flavor-access',
        )

        self.assertEqual(resp.body['flavor_access'], rsp)

    def test_fetch_extra_specs(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        resp = mock.Mock()
        resp.body = {
            'extra_specs':
                {'a': 'b',
                 'c': 'd'}
        }
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.get = mock.Mock(return_value=resp)

        rsp = sot.fetch_extra_specs(self.sess)

        self.sess.get.assert_called_with(
            'flavors/IDENTIFIER/os-extra_specs',
            microversion=self.sess.default_microversion
        )

        self.assertEqual(resp.body['extra_specs'], rsp.extra_specs)
        self.assertIsInstance(rsp, flavor.Flavor)

    def test_create_extra_specs(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        specs = {
            'a': 'b',
            'c': 'd'
        }
        resp = mock.Mock()
        resp.body = {
            'extra_specs': specs
        }
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.post = mock.Mock(return_value=resp)

        rsp = sot.create_extra_specs(self.sess, specs)

        self.sess.post.assert_called_with(
            'flavors/IDENTIFIER/os-extra_specs',
            json={'extra_specs': specs},
            microversion=self.sess.default_microversion
        )

        self.assertEqual(resp.body['extra_specs'], rsp.extra_specs)
        self.assertIsInstance(rsp, flavor.Flavor)

    def test_get_extra_specs_property(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        resp = mock.Mock()
        resp.body = {
            'a': 'b'
        }
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.get = mock.Mock(return_value=resp)

        rsp = sot.get_extra_specs_property(self.sess, 'a')

        self.sess.get.assert_called_with(
            'flavors/IDENTIFIER/os-extra_specs/a',
            microversion=self.sess.default_microversion
        )

        self.assertEqual('b', rsp)

    def test_update_extra_specs_property(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        resp = mock.Mock()
        resp.body = {
            'a': 'b'
        }
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.put = mock.Mock(return_value=resp)

        rsp = sot.update_extra_specs_property(self.sess, 'a', 'b')

        self.sess.put.assert_called_with(
            'flavors/IDENTIFIER/os-extra_specs/a',
            json={'a': 'b'},
            microversion=self.sess.default_microversion
        )

        self.assertEqual('b', rsp)

    def test_delete_extra_specs_property(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        resp = mock.Mock()
        resp.body = None
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.delete = mock.Mock(return_value=resp)

        rsp = sot.delete_extra_specs_property(self.sess, 'a')

        self.sess.delete.assert_called_with(
            'flavors/IDENTIFIER/os-extra_specs/a',
            microversion=self.sess.default_microversion
        )

        self.assertIsNone(rsp)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import proxy
from openstack.workflow.v2 import execution as _execution
from openstack.workflow.v2 import workflow as _workflow


class Proxy(proxy.Proxy):

    def create_workflow(self, **attrs):
        """Create a new workflow from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.workflow.v2.workflow.Workflow`,
            comprised of the properties on the Workflow class.

        :returns: The results of workflow creation
        :rtype: :class:`~openstack.workflow.v2.workflow.Workflow`
        """
        return self._create(_workflow.Workflow, **attrs)

    def get_workflow(self, *attrs):
        """Get a workflow

        :param workflow: The value can be the name of a workflow or
            :class:`~openstack.workflow.v2.workflow.Workflow` instance.

        :returns: One :class:`~openstack.workflow.v2.workflow.Workflow`
        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
            workflow matching the name could be found.
        """
        return self._get(_workflow.Workflow, *attrs)

    def workflows(self, **query):
        """Retrieve a generator of workflows

        :param kwargs query: Optional query parameters to be sent to
            restrict the workflows to be returned. Available parameters
            include:

            * limit: Requests at most the specified number of items be
              returned from the query.
            * marker: Specifies the ID of the last-seen workflow. Use the
              limit parameter to make an initial limited request and use
              the ID of the last-seen workflow from the response as the
              marker parameter value in a subsequent limited request.

        :returns: A generator of workflow instances.
        """
        return self._list(_workflow.Workflow, **query)

    def delete_workflow(self, value, ignore_missing=True):
        """Delete a workflow

        :param value: The value can be either the name of a workflow or a
            :class:`~openstack.workflow.v2.workflow.Workflow`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will
            be raised when the workflow does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent workflow.

        :returns: ``None``
        """
        return self._delete(_workflow.Workflow, value,
                            ignore_missing=ignore_missing)

    def find_workflow(self, name_or_id, ignore_missing=True):
        """Find a single workflow

        :param name_or_id: The name or ID of an workflow.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.compute.v2.workflow.Extension` or
            None
        """
        return self._find(_workflow.Workflow, name_or_id,
                          ignore_missing=ignore_missing)

    def create_execution(self, **attrs):
        """Create a new execution from attributes

        :param workflow_name: The name of target workflow to execute.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.workflow.v2.execution.Execution`,
            comprised of the properties on the Execution class.

        :returns: The results of execution creation
        :rtype: :class:`~openstack.workflow.v2.execution.Execution`
        """
        return self._create(_execution.Execution, **attrs)

    def get_execution(self, *attrs):
        """Get a execution

        :param workflow_name: The name of target workflow to execute.
        :param execution: The value can be either the ID of a execution or a
            :class:`~openstack.workflow.v2.execution.Execution` instance.

        :returns: One :class:`~openstack.workflow.v2.execution.Execution`
        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
            execution matching the criteria could be found.
        """
        return self._get(_execution.Execution, *attrs)

    def executions(self, **query):
        """Retrieve a generator of executions

        :param kwargs query: Optional query parameters to be sent to
            restrict the executions to be returned. Available parameters
            include:

            * limit: Requests at most the specified number of items be
              returned from the query.
            * marker: Specifies the ID of the last-seen execution. Use the
              limit parameter to make an initial limited request and use
              the ID of the last-seen execution from the response as the
              marker parameter value in a subsequent limited request.

        :returns: A generator of execution instances.
        """
        return self._list(_execution.Execution, **query)

    def delete_execution(self, value, ignore_missing=True):
        """Delete an execution

        :param value: The value can be either the name of a execution or a
            :class:`~openstack.workflow.v2.execute.Execution`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the execution does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent execution.

        :returns: ``None``
        """
        return self._delete(_execution.Execution, value,
                            ignore_missing=ignore_missing)

    def find_execution(self, name_or_id, ignore_missing=True):
        """Find a single execution

        :param name_or_id: The name or ID of an execution.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.compute.v2.execution.Execution` or
            None
        """
        return self._find(_execution.Execution, name_or_id,
                          ignore_missing=ignore_missing)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import floating_ip as _fip
from openstack.dns.v2 import recordset as _rs
from openstack.dns.v2 import zone as _zone
from openstack.dns.v2 import zone_export as _zone_export
from openstack.dns.v2 import zone_import as _zone_import
from openstack.dns.v2 import zone_transfer as _zone_transfer
from openstack import proxy


class Proxy(proxy.Proxy):

    # ======== Zones ========
    def zones(self, **query):
        """Retrieve a generator of zones

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `name`: Zone Name field.
            * `type`: Zone Type field.
            * `email`: Zone email field.
            * `status`: Status of the zone.
            * `ttl`: TTL field filter.abs
            * `description`: Zone description field filter.

        :returns: A generator of zone
            :class:`~openstack.dns.v2.zone.Zone` instances.
        """
        return self._list(_zone.Zone, **query)

    def create_zone(self, **attrs):
        """Create a new zone from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.dns.v2.zone.Zone`,
            comprised of the properties on the Zone class.
        :returns: The results of zone creation.
        :rtype: :class:`~openstack.dns.v2.zone.Zone`
        """
        return self._create(_zone.Zone, prepend_key=False, **attrs)

    def get_zone(self, zone):
        """Get a zone

        :param zone: The value can be the ID of a zone
            or a :class:`~openstack.dns.v2.zone.Zone` instance.
        :returns: Zone instance.
        :rtype: :class:`~openstack.dns.v2.zone.Zone`
        """
        return self._get(_zone.Zone, zone)

    def delete_zone(self, zone, ignore_missing=True):
        """Delete a zone

        :param zone: The value can be the ID of a zone
            or a :class:`~openstack.dns.v2.zone.Zone` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the zone does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent zone.

        :returns: Zone been deleted
        :rtype: :class:`~openstack.dns.v2.zone.Zone`
        """
        return self._delete(_zone.Zone, zone, ignore_missing=ignore_missing)

    def update_zone(self, zone, **attrs):
        """Update zone attributes

        :param zone: The id or an instance of
            :class:`~openstack.dns.v2.zone.Zone`.
        :param dict attrs: attributes for update on
            :class:`~openstack.dns.v2.zone.Zone`.

        :rtype: :class:`~openstack.dns.v2.zone.Zone`
        """
        return self._update(_zone.Zone, zone, **attrs)

    def find_zone(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single zone

        :param name_or_id: The name or ID of a zone
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the zone does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent zone.

        :returns: :class:`~openstack.dns.v2.zone.Zone`
        """
        return self._find(_zone.Zone, name_or_id,
                          ignore_missing=ignore_missing)

    def abandon_zone(self, zone, **attrs):
        """Abandon Zone

        :param zone: The value can be the ID of a zone to be abandoned
            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.

        :returns: None
        """
        zone = self._get_resource(_zone.Zone, zone)

        return zone.abandon(self)

    def xfr_zone(self, zone, **attrs):
        """Trigger update of secondary Zone

        :param zone: The value can be the ID of a zone to be abandoned
            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.

        :returns: None
        """
        zone = self._get_resource(_zone.Zone, zone)
        return zone.xfr(self)

    # ======== Recordsets ========
    def recordsets(self, zone=None, **query):
        """Retrieve a generator of recordsets

        :param zone: The optional value can be the ID of a zone
            or a :class:`~openstack.dns.v2.zone.Zone` instance. If it is not
            given all recordsets for all zones of the tenant would be
            retrieved
        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `name`: Recordset Name field.
            * `type`: Type field.
            * `status`: Status of the recordset.
            * `ttl`: TTL field filter.
            * `description`: Recordset description field filter.

        :returns: A generator of zone
            (:class:`~openstack.dns.v2.recordset.Recordset`) instances
        """
        base_path = None
        if not zone:
            base_path = '/recordsets'
        else:
            zone = self._get_resource(_zone.Zone, zone)
            query.update({'zone_id': zone.id})
        return self._list(_rs.Recordset, base_path=base_path, **query)

    def create_recordset(self, zone, **attrs):
        """Create a new recordset in the zone

        :param zone: The value can be the ID of a zone
            or a :class:`~openstack.dns.v2.zone.Zone` instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.dns.v2.recordset.Recordset`,
            comprised of the properties on the Recordset class.
        :returns: The results of zone creation
        :rtype: :class:`~openstack.dns.v2.recordset.Recordset`
        """
        zone = self._get_resource(_zone.Zone, zone)
        attrs.update({'zone_id': zone.id})
        return self._create(_rs.Recordset, prepend_key=False, **attrs)

    def update_recordset(self, recordset, **attrs):
        """Update Recordset attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.dns.v2.recordset.Recordset`,
            comprised of the properties on the Recordset class.
        :returns: The results of zone creation
        :rtype: :class:`~openstack.dns.v2.recordset.Recordset`
        """
        return self._update(_rs.Recordset, recordset, **attrs)

    def get_recordset(self, recordset, zone):
        """Get a recordset

        :param zone: The value can be the ID of a zone
            or a :class:`~openstack.dns.v2.zone.Zone` instance.
        :param recordset: The value can be the ID of a recordset
            or a :class:`~openstack.dns.v2.recordset.Recordset` instance.
        :returns: Recordset instance
        :rtype: :class:`~openstack.dns.v2.recordset.Recordset`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return self._get(_rs.Recordset, recordset, zone_id=zone.id)

    def delete_recordset(self, recordset, zone=None, ignore_missing=True):
        """Delete a zone

        :param recordset: The value can be the ID of a recordset
            or a :class:`~openstack.dns.v2.recordset.Recordset`
            instance.
        :param zone: The value can be the ID of a zone
            or a :class:`~openstack.dns.v2.zone.Zone` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the zone does not exist. When set to ``True``, no exception will
            be set when attempting to delete a nonexistent zone.

        :returns: Recordset instance been deleted
        :rtype: :class:`~openstack.dns.v2.recordset.Recordset`
        """
        if zone:
            zone = self._get_resource(_zone.Zone, zone)
            recordset = self._get(
                _rs.Recordset, recordset, zone_id=zone.id)
        return self._delete(_rs.Recordset, recordset,
                            ignore_missing=ignore_missing)

    def find_recordset(self, zone, name_or_id, ignore_missing=True, **attrs):
        """Find a single recordset

        :param zone: The value can be the ID of a zone
            or a :class:`~openstack.dns.v2.zone.Zone` instance.
        :param name_or_id: The name or ID of a zone
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the zone does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent zone.

        :returns: :class:`~openstack.dns.v2.recordset.Recordset`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return self._find(_rs.Recordset, name_or_id,
                          ignore_missing=ignore_missing, zone_id=zone.id,
                          **attrs)

    # ======== Zone Imports ========
    def zone_imports(self, **query):
        """Retrieve a generator of zone imports

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `zone_id`: Zone I field.
            * `message`: Message field.
            * `status`: Status of the zone import record.

        :returns: A generator of zone
            :class:`~openstack.dns.v2.zone_import.ZoneImport` instances.
        """
        return self._list(_zone_import.ZoneImport, **query)

    def create_zone_import(self, **attrs):
        """Create a new zone import from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.dns.v2.zone_import.ZoneImport`,
            comprised of the properties on the ZoneImport class.
        :returns: The results of zone creation.
        :rtype: :class:`~openstack.dns.v2.zone_import.ZoneImport`
        """
        return self._create(_zone_import.ZoneImport, prepend_key=False,
                            **attrs)

    def get_zone_import(self, zone_import):
        """Get a zone import record

        :param zone: The value can be the ID of a zone import
            or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
        :returns: ZoneImport instance.
        :rtype: :class:`~openstack.dns.v2.zone_import.ZoneImport`
        """
        return self._get(_zone_import.ZoneImport, zone_import)

    def delete_zone_import(self, zone_import, ignore_missing=True):
        """Delete a zone import

        :param zone_import: The value can be the ID of a zone import
            or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the zone does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent zone.

        :returns: None
        """
        return self._delete(_zone_import.ZoneImport, zone_import,
                            ignore_missing=ignore_missing)

    # ======== Zone Exports ========
    def zone_exports(self, **query):
        """Retrieve a generator of zone exports

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `zone_id`: Zone I field.
            * `message`: Message field.
            * `status`: Status of the zone import record.

        :returns: A generator of zone
            :class:`~openstack.dns.v2.zone_export.ZoneExport` instances.
        """
        return self._list(_zone_export.ZoneExport, **query)

    def create_zone_export(self, zone, **attrs):
        """Create a new zone export from attributes

        :param zone: The value can be the ID of a zone to be exported
            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.dns.v2.zone_export.ZoneExport`,
            comprised of the properties on the ZoneExport class.
        :returns: The results of zone creation.
        :rtype: :class:`~openstack.dns.v2.zone_export.ZoneExport`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return self._create(_zone_export.ZoneExport,
                            base_path='/zones/%(zone_id)s/tasks/export',
                            prepend_key=False,
                            zone_id=zone.id,
                            **attrs)

    def get_zone_export(self, zone_export):
        """Get a zone export record

        :param zone: The value can be the ID of a zone import
            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
        :returns: ZoneExport instance.
        :rtype: :class:`~openstack.dns.v2.zone_export.ZoneExport`
        """
        return self._get(_zone_export.ZoneExport, zone_export)

    def get_zone_export_text(self, zone_export):
        """Get a zone export record as text

        :param zone: The value can be the ID of a zone import
            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
        :returns: ZoneExport instance.
        :rtype: :class:`~openstack.dns.v2.zone_export.ZoneExport`
        """
        return self._get(_zone_export.ZoneExport, zone_export,
                         base_path='/zones/tasks/export/%(id)s/export')

    def delete_zone_export(self, zone_export, ignore_missing=True):
        """Delete a zone export

        :param zone_export: The value can be the ID of a zone import
            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the zone does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent zone.

        :returns: None
        """
        return self._delete(_zone_export.ZoneExport, zone_export,
                            ignore_missing=ignore_missing)

    # ======== FloatingIPs ========
    def floating_ips(self, **query):
        """Retrieve a generator of recordsets

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `name`: Recordset Name field.
            * `type`: Type field.
            * `status`: Status of the recordset.
            * `ttl`: TTL field filter.
            * `description`: Recordset description field filter.

        :returns: A generator of floatingips
            (:class:`~openstack.dns.v2.floating_ip.FloatingIP`) instances
        """
        return self._list(_fip.FloatingIP, **query)

    def get_floating_ip(self, floating_ip):
        """Get a Floating IP

        :param floating_ip: The value can be the ID of a floating ip
            or a :class:`~openstack.dns.v2.floating_ip.FloatingIP` instance.
            The ID is in format "region_name:floatingip_id"
        :returns: FloatingIP instance.
        :rtype: :class:`~openstack.dns.v2.floating_ip.FloatingIP`
        """
        return self._get(_fip.FloatingIP, floating_ip)

    def update_floating_ip(self, floating_ip, **attrs):
        """Update floating ip attributes

        :param floating_ip: The id or an instance of
            :class:`~openstack.dns.v2.fip.FloatingIP`.
        :param dict attrs: attributes for update on
            :class:`~openstack.dns.v2.fip.FloatingIP`.

        :rtype: :class:`~openstack.dns.v2.fip.FloatingIP`
        """
        return self._update(_fip.FloatingIP, floating_ip, **attrs)

    def unset_floating_ip(self, floating_ip):
        """Unset a Floating IP PTR record
        :param floating_ip: ID for the floatingip associated with the
            project.
        :returns: FloatingIP PTR record.
        :rtype: :class:`~openstack.dns.v2.fip.FloatipgIP`
        """
        # concat `region:floating_ip_id` as id
        attrs = {'ptrdname': None}
        return self._update(_fip.FloatingIP, floating_ip, **attrs)

    # ======== Zone Transfer ========
    def zone_transfer_requests(self, **query):
        """Retrieve a generator of zone transfer requests

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `status`: Status of the recordset.

        :returns: A generator of transfer requests
            (:class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`)
            instances
        """
        return self._list(_zone_transfer.ZoneTransferRequest, **query)

    def get_zone_transfer_request(self, request):
        """Get a ZoneTransfer Request info

        :param request: The value can be the ID of a transfer request
            or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
            instance.
        :returns: Zone transfer request instance.
        :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
        """
        return self._get(_zone_transfer.ZoneTransferRequest, request)

    def create_zone_transfer_request(self, zone, **attrs):
        """Create a new ZoneTransfer Request from attributes

        :param zone: The value can be the ID of a zone to be transferred
            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`,
            comprised of the properties on the ZoneTransferRequest class.
        :returns: The results of zone transfer request creation.
        :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return self._create(
            _zone_transfer.ZoneTransferRequest,
            base_path='/zones/%(zone_id)s/tasks/transfer_requests',
            prepend_key=False,
            zone_id=zone.id,
            **attrs)

    def update_zone_transfer_request(self, request, **attrs):
        """Update ZoneTransfer Request attributes

        :param floating_ip: The id or an instance of
            :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`.
        :param dict attrs: attributes for update on
            :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`.

        :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
        """
        return self._update(_zone_transfer.ZoneTransferRequest,
                            request, **attrs)

    def delete_zone_transfer_request(self, request, ignore_missing=True):
        """Delete a ZoneTransfer Request

        :param request: The value can be the ID of a zone transfer request
            or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the zone does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent zone.

        :returns: None
        """
        return self._delete(_zone_transfer.ZoneTransferRequest, request,
                            ignore_missing=ignore_missing)

    def zone_transfer_accepts(self, **query):
        """Retrieve a generator of zone transfer accepts

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `status`: Status of the recordset.

        :returns: A generator of transfer accepts
            (:class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`)
            instances
        """
        return self._list(_zone_transfer.ZoneTransferAccept, **query)

    def get_zone_transfer_accept(self, accept):
        """Get a ZoneTransfer Accept info

        :param request: The value can be the ID of a transfer accept
            or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`
            instance.
        :returns: Zone transfer request instance.
        :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`
        """
        return self._get(_zone_transfer.ZoneTransferAccept, accept)

    def create_zone_transfer_accept(self, **attrs):
        """Create a new ZoneTransfer Accept from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`,
            comprised of the properties on the ZoneTransferAccept class.
        :returns: The results of zone transfer request creation.
        :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`
        """
        return self._create(_zone_transfer.ZoneTransferAccept, **attrs)

    def _get_cleanup_dependencies(self):
        # DNS may depend on floating ip
        return {
            'dns': {
                'before': ['network']
            }
        }

    def _service_cleanup(self, dry_run=True, client_status_queue=False,
                         identified_resources=None,
                         filters=None, resource_evaluation_fn=None):
        # Delete all zones
        for obj in self.zones():
            self._service_cleanup_del_res(
                self.delete_zone,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)
        # Unset all floatingIPs
        # NOTE: FloatingIPs are not cleaned when filters are set
        for obj in self.floating_ips():
            self._service_cleanup_del_res(
                self.unset_floating_ip,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from openstack.clustering.v1 import cluster
from openstack.tests.functional import base
from openstack.tests.functional.network.v2 import test_network


class TestCluster(base.BaseFunctionalTest):

    _wait_for_timeout_key = 'OPENSTACKSDK_FUNC_TEST_TIMEOUT_CLUSTER'

    def setUp(self):
        super(TestCluster, self).setUp()
        self.require_service('clustering')

        self.cidr = '10.99.99.0/16'

        self.network, self.subnet = test_network.create_network(
            self.conn,
            self.getUniqueString(),
            self.cidr)
        self.assertIsNotNone(self.network)

        profile_attrs = {
            'name': self.getUniqueString(),
            'spec': {
                'type': 'os.nova.server',
                'version': 1.0,
                'properties': {
                    'name': self.getUniqueString(),
                    'flavor': self.flavor.name,
                    'image': self.image.name,
                    'networks': [{'network': self.network.id}]
                }}}

        self.profile = self.conn.clustering.create_profile(**profile_attrs)
        self.assertIsNotNone(self.profile)

        self.cluster_name = self.getUniqueString()
        cluster_spec = {
            "name": self.cluster_name,
            "profile_id": self.profile.name,
            "min_size": 0,
            "max_size": -1,
            "desired_capacity": 0,
        }

        self.cluster = self.conn.clustering.create_cluster(**cluster_spec)
        self.conn.clustering.wait_for_status(
            self.cluster, 'ACTIVE',
            wait=self._wait_for_timeout)
        assert isinstance(self.cluster, cluster.Cluster)

    def tearDown(self):
        if self.cluster:
            self.conn.clustering.delete_cluster(self.cluster.id)
            self.conn.clustering.wait_for_delete(
                self.cluster, wait=self._wait_for_timeout)

        test_network.delete_network(self.conn, self.network, self.subnet)

        self.conn.clustering.delete_profile(self.profile)

        super(TestCluster, self).tearDown()

    def test_find(self):
        sot = self.conn.clustering.find_cluster(self.cluster.id)
        self.assertEqual(self.cluster.id, sot.id)

    def test_get(self):
        sot = self.conn.clustering.get_cluster(self.cluster)
        self.assertEqual(self.cluster.id, sot.id)

    def test_list(self):
        names = [o.name for o in self.conn.clustering.clusters()]
        self.assertIn(self.cluster_name, names)

    def test_update(self):
        new_cluster_name = self.getUniqueString()
        sot = self.conn.clustering.update_cluster(
            self.cluster, name=new_cluster_name, profile_only=False)

        time.sleep(2)
        sot = self.conn.clustering.get_cluster(self.cluster)
        self.assertEqual(new_cluster_name, sot.name)

    def test_delete(self):
        cluster_delete_action = self.conn.clustering.delete_cluster(
            self.cluster.id)

        self.conn.clustering.wait_for_delete(self.cluster,
                                             wait=self._wait_for_timeout)

        action = self.conn.clustering.get_action(cluster_delete_action.id)
        self.assertEqual(action.target_id, self.cluster.id)
        self.assertEqual(action.action, 'CLUSTER_DELETE')
        self.assertEqual(action.status, 'SUCCEEDED')

        self.cluster = None

    def test_force_delete(self):
        cluster_delete_action = self.conn.clustering.delete_cluster(
            self.cluster.id, False, True)

        self.conn.clustering.wait_for_delete(self.cluster,
                                             wait=self._wait_for_timeout)

        action = self.conn.clustering.get_action(cluster_delete_action.id)
        self.assertEqual(action.target_id, self.cluster.id)
        self.assertEqual(action.action, 'CLUSTER_DELETE')
        self.assertEqual(action.status, 'SUCCEEDED')

        self.cluster = None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource as _resource


class AvailabilityZone(_resource.Resource):
    resource_key = 'availability_zone'
    resources_key = 'availability_zones'
    base_path = '/availability_zones'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = False
    allow_fetch = False
    allow_commit = False
    allow_delete = False
    allow_list = True

    # NOTE: We don't support query by state yet because there is a mapping
    #       at neutron side difficult to map.
    _query_mapping = _resource.QueryParameters(
        name='availability_zone', resource='agent_type')

    # Properties
    #: Name of the availability zone.
    name = _resource.Body('name')
    #: Type of resource for the availability zone, such as ``network``.
    resource = _resource.Body('resource')
    #: State of the availability zone, either ``available`` or
    #: ``unavailable``.
    state = _resource.Body('state')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v1 import image
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'checksum': '1',
    'container_format': '2',
    'copy_from': '3',
    'disk_format': '4',
    'id': IDENTIFIER,
    'is_public': True,
    'location': '6',
    'min_disk': '7',
    'min_ram': '8',
    'name': '9',
    'owner': '10',
    'properties': '11',
    'protected': True,
    'size': '13',
    'status': '14',
    'created_at': '2015-03-09T12:14:57.233772',
    'updated_at': '2015-03-09T12:15:57.233772',
}


class TestImage(base.TestCase):

    def test_basic(self):
        sot = image.Image()
        self.assertEqual('image', sot.resource_key)
        self.assertEqual('images', sot.resources_key)
        self.assertEqual('/images', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = image.Image(**EXAMPLE)
        self.assertEqual(EXAMPLE['checksum'], sot.checksum)
        self.assertEqual(EXAMPLE['container_format'], sot.container_format)
        self.assertEqual(EXAMPLE['copy_from'], sot.copy_from)
        self.assertEqual(EXAMPLE['disk_format'], sot.disk_format)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertTrue(sot.is_public)
        self.assertEqual(EXAMPLE['location'], sot.location)
        self.assertEqual(EXAMPLE['min_disk'], sot.min_disk)
        self.assertEqual(EXAMPLE['min_ram'], sot.min_ram)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['owner'], sot.owner)
        self.assertEqual(EXAMPLE['owner'], sot.owner_id)
        self.assertEqual(EXAMPLE['properties'], sot.properties)
        self.assertTrue(sot.is_protected)
        self.assertEqual(EXAMPLE['size'], sot.size)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from openstack.orchestration.v1 import stack_template
from openstack.tests.unit import base


FAKE = {
    'description': 'template description',
    'heat_template_version': '2014-10-16',
    'parameters': {
        'key_name': {
            'type': 'string'
        }
    },
    'resources': {
        'resource1': {
            'type': 'ResourceType'
        }
    },
    'conditions': {'cd1': True},
    'outputs': {
        'key1': 'value1'
    }
}


class TestStackTemplate(base.TestCase):

    def test_basic(self):
        sot = stack_template.StackTemplate()
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = stack_template.StackTemplate(**FAKE)
        self.assertEqual(FAKE['description'], sot.description)
        self.assertEqual(FAKE['heat_template_version'],
                         sot.heat_template_version)
        self.assertEqual(FAKE['outputs'], sot.outputs)
        self.assertEqual(FAKE['parameters'], sot.parameters)
        self.assertEqual(FAKE['resources'], sot.resources)
        self.assertEqual(FAKE['conditions'], sot.conditions)

    def test_to_dict(self):
        fake_sot = copy.deepcopy(FAKE)
        fake_sot['parameter_groups'] = [{
            "description": "server parameters",
            "parameters": ["key_name", "image_id"],
            "label": "server_parameters"}]
        fake_sot['location'] = None
        fake_sot['id'] = None
        fake_sot['name'] = None

        for temp_version in ['2016-10-14', '2017-02-24', '2017-02-24',
                             '2017-09-01', '2018-03-02', 'newton',
                             'ocata', 'pike', 'queens']:
            fake_sot['heat_template_version'] = temp_version
            sot = stack_template.StackTemplate(**fake_sot)
            self.assertEqual(fake_sot, sot.to_dict())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.network.v2 import floating_ip
from openstack import proxy
from openstack.tests.unit import base

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'created_at': '0',
    'fixed_ip_address': '1',
    'floating_ip_address': '127.0.0.1',
    'floating_network_id': '3',
    'id': IDENTIFIER,
    'port_id': '5',
    'qos_policy_id': '51',
    'project_id': '6',
    'router_id': '7',
    'description': '8',
    'dns_domain': '9',
    'dns_name': '10',
    'status': 'ACTIVE',
    'revision_number': 12,
    'updated_at': '13',
    'subnet_id': '14',
    'tags': ['15', '16']
}


class TestFloatingIP(base.TestCase):

    def test_basic(self):
        sot = floating_ip.FloatingIP()
        self.assertEqual('floatingip', sot.resource_key)
        self.assertEqual('floatingips', sot.resources_key)
        self.assertEqual('/floatingips', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = floating_ip.FloatingIP(**EXAMPLE)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['fixed_ip_address'], sot.fixed_ip_address)
        self.assertEqual(EXAMPLE['floating_ip_address'],
                         sot.floating_ip_address)
        self.assertEqual(EXAMPLE['floating_network_id'],
                         sot.floating_network_id)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['port_id'], sot.port_id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['router_id'], sot.router_id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['dns_domain'], sot.dns_domain)
        self.assertEqual(EXAMPLE['dns_name'], sot.dns_name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['subnet_id'], sot.subnet_id)
        self.assertEqual(EXAMPLE['tags'], sot.tags)

        self.assertDictEqual(
            {'limit': 'limit',
             'marker': 'marker',
             'description': 'description',
             'project_id': 'project_id',
             'tenant_id': 'project_id',
             'status': 'status',
             'port_id': 'port_id',
             'subnet_id': 'subnet_id',
             'router_id': 'router_id',
             'fixed_ip_address': 'fixed_ip_address',
             'floating_ip_address': 'floating_ip_address',
             'floating_network_id': 'floating_network_id',
             'tags': 'tags',
             'any_tags': 'tags-any',
             'not_tags': 'not-tags',
             'not_any_tags': 'not-tags-any',
             },
            sot._query_mapping._mapping)

    def test_find_available(self):
        mock_session = mock.Mock(spec=proxy.Proxy)
        mock_session.get_filter = mock.Mock(return_value={})
        mock_session.default_microversion = None
        mock_session.session = self.cloud.session
        data = {'id': 'one', 'floating_ip_address': '10.0.0.1'}
        fake_response = mock.Mock()
        body = {floating_ip.FloatingIP.resources_key: [data]}
        fake_response.json = mock.Mock(return_value=body)
        fake_response.status_code = 200
        mock_session.get = mock.Mock(return_value=fake_response)

        result = floating_ip.FloatingIP.find_available(mock_session)

        self.assertEqual('one', result.id)
        mock_session.get.assert_called_with(
            floating_ip.FloatingIP.base_path,
            headers={'Accept': 'application/json'},
            params={},
            microversion=None)

    def test_find_available_nada(self):
        mock_session = mock.Mock(spec=proxy.Proxy)
        mock_session.default_microversion = None
        fake_response = mock.Mock()
        body = {floating_ip.FloatingIP.resources_key: []}
        fake_response.json = mock.Mock(return_value=body)
        fake_response.status_code = 200
        mock_session.get = mock.Mock(return_value=fake_response)

        self.assertIsNone(floating_ip.FloatingIP.find_available(mock_session))



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from openstack.config.loader import OpenStackConfig  # noqa


def get_cloud_region(
        service_key=None, options=None,
        app_name=None, app_version=None,
        load_yaml_config=True,
        load_envvars=True,
        **kwargs):
    config = OpenStackConfig(
        load_yaml_config=load_yaml_config,
        load_envvars=load_envvars,
        app_name=app_name, app_version=app_version)
    if options:
        config.register_argparse_arguments(options, sys.argv, service_key)
        parsed_options = options.parse_known_args(sys.argv)
    else:
        parsed_options = None

    return config.get_one(options=parsed_options, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import flavor
from openstack.tests.functional import base


class TestFlavor(base.BaseFunctionalTest):

    UPDATE_NAME = "UPDATED-NAME"
    SERVICE_TYPE = "FLAVORS"
    ID = None

    SERVICE_PROFILE_DESCRIPTION = "DESCRIPTION"
    METAINFO = "FlAVOR_PROFILE_METAINFO"

    def setUp(self):
        super(TestFlavor, self).setUp()
        self.FLAVOR_NAME = self.getUniqueString('flavor')
        flavors = self.conn.network.create_flavor(
            name=self.FLAVOR_NAME,
            service_type=self.SERVICE_TYPE)
        assert isinstance(flavors, flavor.Flavor)
        self.assertEqual(self.FLAVOR_NAME, flavors.name)
        self.assertEqual(self.SERVICE_TYPE, flavors.service_type)

        self.ID = flavors.id

        self.service_profiles = self.conn.network.create_service_profile(
            description=self.SERVICE_PROFILE_DESCRIPTION,
            metainfo=self.METAINFO,)

    def tearDown(self):
        flavors = self.conn.network.delete_flavor(self.ID, ignore_missing=True)
        self.assertIsNone(flavors)

        service_profiles = self.conn.network.delete_service_profile(
            self.ID, ignore_missing=True)
        self.assertIsNone(service_profiles)
        super(TestFlavor, self).tearDown()

    def test_find(self):
        flavors = self.conn.network.find_flavor(self.FLAVOR_NAME)
        self.assertEqual(self.ID, flavors.id)

    def test_get(self):
        flavors = self.conn.network.get_flavor(self.ID)
        self.assertEqual(self.FLAVOR_NAME, flavors.name)
        self.assertEqual(self.ID, flavors.id)

    def test_list(self):
        names = [f.name for f in self.conn.network.flavors()]
        self.assertIn(self.FLAVOR_NAME, names)

    def test_update(self):
        flavor = self.conn.network.update_flavor(self.ID,
                                                 name=self.UPDATE_NAME)
        self.assertEqual(self.UPDATE_NAME, flavor.name)

    def test_associate_disassociate_flavor_with_service_profile(self):
        response = \
            self.conn.network.associate_flavor_with_service_profile(
                self.ID, self.service_profiles.id)
        self.assertIsNotNone(response)

        response = \
            self.conn.network.disassociate_flavor_from_service_profile(
                self.ID, self.service_profiles.id)
        self.assertIsNone(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from keystoneauth1 import adapter

from openstack.common import metadata
from openstack import exceptions
from openstack import resource
from openstack.tests.unit import base
from openstack.tests.unit.test_resource import FakeResponse

IDENTIFIER = 'IDENTIFIER'


class TestMetadata(base.TestCase):

    def setUp(self):
        super(TestMetadata, self).setUp()

        self.service_name = "service"
        self.base_path = "base_path"

        self.metadata_result = {"metadata": {"go": "cubs", "boo": "sox"}}
        self.meta_result = {"meta": {"oh": "yeah"}}

        class Test(resource.Resource, metadata.MetadataMixin):
            service = self.service_name
            base_path = self.base_path
            resources_key = 'resources'
            allow_create = True
            allow_fetch = True
            allow_head = True
            allow_commit = True
            allow_delete = True
            allow_list = True

        self.test_class = Test

        self.request = mock.Mock(spec=resource._Request)
        self.request.url = "uri"
        self.request.body = "body"
        self.request.headers = "headers"

        self.response = FakeResponse({})

        self.sot = Test.new(id="id")
        self.sot._prepare_request = mock.Mock(return_value=self.request)
        self.sot._translate_response = mock.Mock()

        self.session = mock.Mock(spec=adapter.Adapter)
        self.session.get = mock.Mock(return_value=self.response)
        self.session.put = mock.Mock(return_value=self.response)
        self.session.post = mock.Mock(return_value=self.response)
        self.session.delete = mock.Mock(return_value=self.response)

    def test_metadata_attribute(self):
        res = self.sot
        self.assertTrue(hasattr(res, 'metadata'))

    def test_get_metadata(self):
        res = self.sot

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.links = {}
        mock_response.json.return_value = {'metadata': {'foo': 'bar'}}

        self.session.get.side_effect = [mock_response]

        result = res.fetch_metadata(self.session)
        # Check metadata attribute is updated
        self.assertDictEqual({'foo': 'bar'}, result.metadata)
        # Check passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata'
        self.session.get.assert_called_once_with(url)

    def test_set_metadata(self):
        res = self.sot

        result = res.set_metadata(self.session, {'foo': 'bar'})
        # Check metadata attribute is updated
        self.assertDictEqual({'foo': 'bar'}, res.metadata)
        # Check passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata'
        self.session.post.assert_called_once_with(
            url,
            json={'metadata': {'foo': 'bar'}}
        )

    def test_replace_metadata(self):
        res = self.sot

        result = res.replace_metadata(self.session, {'foo': 'bar'})
        # Check metadata attribute is updated
        self.assertDictEqual({'foo': 'bar'}, res.metadata)
        # Check passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata'
        self.session.put.assert_called_once_with(
            url,
            json={'metadata': {'foo': 'bar'}}
        )

    def test_delete_all_metadata(self):
        res = self.sot

        # Set some initial value to check removal
        res.metadata = {'foo': 'bar'}

        result = res.delete_metadata(self.session)
        # Check metadata attribute is updated
        self.assertEqual({}, res.metadata)
        # Check passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata'
        self.session.put.assert_called_once_with(
            url,
            json={'metadata': {}})

    def test_get_metadata_item(self):
        res = self.sot

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'meta': {'foo': 'bar'}}
        self.session.get.side_effect = [mock_response]

        result = res.get_metadata_item(self.session, 'foo')
        # Check tags attribute is updated
        self.assertEqual({'foo': 'bar'}, res.metadata)
        # Check the passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata/foo'
        self.session.get.assert_called_once_with(url)

    def test_delete_single_item(self):
        res = self.sot

        res.metadata = {'foo': 'bar', 'foo2': 'bar2'}

        result = res.delete_metadata_item(self.session, 'foo2')
        # Check metadata attribute is updated
        self.assertEqual({'foo': 'bar'}, res.metadata)
        # Check passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata/foo2'
        self.session.delete.assert_called_once_with(url)

    def test_delete_signle_item_empty(self):
        res = self.sot

        result = res.delete_metadata_item(self.session, 'foo2')
        # Check metadata attribute is updated
        self.assertEqual({}, res.metadata)
        # Check passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata/foo2'
        self.session.delete.assert_called_once_with(url)

    def test_get_metadata_item_not_exists(self):
        res = self.sot

        mock_response = mock.Mock()
        mock_response.status_code = 404
        mock_response.content = None
        self.session.get.side_effect = [mock_response]

        # ensure we get 404
        self.assertRaises(
            exceptions.NotFoundException,
            res.get_metadata_item,
            self.session,
            'dummy',
        )

    def test_set_metadata_item(self):
        res = self.sot

        # Set some initial value to check add
        res.metadata = {'foo': 'bar'}

        result = res.set_metadata_item(self.session, 'foo', 'black')
        # Check metadata attribute is updated
        self.assertEqual({'foo': 'black'}, res.metadata)
        # Check passed resource is returned
        self.assertEqual(res, result)
        url = self.base_path + '/' + res.id + '/metadata/foo'
        self.session.put.assert_called_once_with(
            url,
            json={'meta': {'foo': 'black'}})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import concurrent
import time

import testtools
from testscenarios import load_tests_apply_scenarios as load_tests  # noqa

import openstack
from openstack.block_storage.v3 import volume as _volume
import openstack.cloud
from openstack.cloud import meta
from openstack.compute.v2 import flavor as _flavor
from openstack import exceptions
from openstack.identity.v3 import project as _project
from openstack.identity.v3 import user as _user
from openstack.image.v2 import image as _image
from openstack.network.v2 import port as _port
from openstack.tests import fakes
from openstack.tests.unit import base
from openstack.tests.unit.cloud import test_port


# Mock out the gettext function so that the task schema can be copypasta
def _(msg):
    return msg


_TASK_PROPERTIES = {
    "id": {
        "description": _("An identifier for the task"),
        "pattern": _('^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}'
                     '-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$'),
        "type": "string"
    },
    "type": {
        "description": _("The type of task represented by this content"),
        "enum": [
            "import",
        ],
        "type": "string"
    },
    "status": {
        "description": _("The current status of this task"),
        "enum": [
            "pending",
            "processing",
            "success",
            "failure"
        ],
        "type": "string"
    },
    "input": {
        "description": _("The parameters required by task, JSON blob"),
        "type": ["null", "object"],
    },
    "result": {
        "description": _("The result of current task, JSON blob"),
        "type": ["null", "object"],
    },
    "owner": {
        "description": _("An identifier for the owner of this task"),
        "type": "string"
    },
    "message": {
        "description": _("Human-readable informative message only included"
                         " when appropriate (usually on failure)"),
        "type": "string",
    },
    "expires_at": {
        "description": _("Datetime when this resource would be"
                         " subject to removal"),
        "type": ["null", "string"]
    },
    "created_at": {
        "description": _("Datetime when this resource was created"),
        "type": "string"
    },
    "updated_at": {
        "description": _("Datetime when this resource was updated"),
        "type": "string"
    },
    'self': {'type': 'string'},
    'schema': {'type': 'string'}
}
_TASK_SCHEMA = dict(
    name='Task', properties=_TASK_PROPERTIES,
    additionalProperties=False,
)


class TestMemoryCache(base.TestCase):

    def setUp(self):
        super(TestMemoryCache, self).setUp(
            cloud_config_fixture='clouds_cache.yaml')

    def _compare_images(self, exp, real):
        self.assertDictEqual(
            _image.Image(**exp).to_dict(computed=False),
            real.to_dict(computed=False))

    def _compare_volumes(self, exp, real):
        self.assertDictEqual(
            _volume.Volume(**exp).to_dict(computed=False),
            real.to_dict(computed=False)
        )

    def test_openstack_cloud(self):
        self.assertIsInstance(self.cloud, openstack.connection.Connection)

    def _compare_projects(self, exp, real):
        self.assertDictEqual(
            _project.Project(**exp).to_dict(computed=False),
            real.to_dict(computed=False)
        )

    def _compare_users(self, exp, real):
        self.assertDictEqual(
            _user.User(**exp).to_dict(computed=False),
            real.to_dict(computed=False)
        )

    def test_list_projects_v3(self):
        project_one = self._get_project_data()
        project_two = self._get_project_data()
        project_list = [project_one, project_two]

        first_response = {'projects': [project_one.json_response['project']]}
        second_response = {'projects': [p.json_response['project']
                                        for p in project_list]}

        mock_uri = self.get_mock_url(
            service_type='identity', resource='projects',
            base_url_append='v3')

        self.register_uris([
            dict(method='GET', uri=mock_uri, status_code=200,
                 json=first_response),
            dict(method='GET', uri=mock_uri, status_code=200,
                 json=second_response)])

        for a, b in zip(first_response['projects'],
                        self.cloud.list_projects()):
            self._compare_projects(a, b)

        # invalidate the list_projects cache
        self.cloud.list_projects.invalidate(self.cloud)

        for a, b in zip(second_response['projects'],
                        self.cloud.list_projects()):
            self._compare_projects(a, b)

        self.assert_calls()

    def test_list_servers_no_herd(self):
        self.cloud._SERVER_AGE = 2
        fake_server = fakes.make_fake_server('1234', 'name')
        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [fake_server]}),
        ])
        with concurrent.futures.ThreadPoolExecutor(16) as pool:
            for i in range(16):
                pool.submit(lambda: self.cloud.list_servers(bare=True))
                # It's possible to race-condition 16 threads all in the
                # single initial lock without a tiny sleep
                time.sleep(0.001)

        self.assert_calls()

    def test_list_volumes(self):
        fake_volume = fakes.FakeVolume('volume1', 'available',
                                       'Volume 1 Display Name')
        fake_volume_dict = meta.obj_to_munch(fake_volume)
        fake_volume2 = fakes.FakeVolume('volume2', 'available',
                                        'Volume 2 Display Name')
        fake_volume2_dict = meta.obj_to_munch(fake_volume2)
        self.register_uris([
            self.get_cinder_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', 'detail']),
                 json={'volumes': [fake_volume_dict]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', 'detail']),
                 json={'volumes': [fake_volume_dict, fake_volume2_dict]})])

        for a, b in zip([fake_volume_dict],
                        self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        # this call should hit the cache
        for a, b in zip([fake_volume_dict],
                        self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        self.cloud.list_volumes.invalidate(self.cloud)
        for a, b in zip([fake_volume_dict, fake_volume2_dict],
                        self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        self.assert_calls()

    def test_list_volumes_creating_invalidates(self):
        fake_volume = fakes.FakeVolume('volume1', 'creating',
                                       'Volume 1 Display Name')
        fake_volume_dict = meta.obj_to_munch(fake_volume)
        fake_volume2 = fakes.FakeVolume('volume2', 'available',
                                        'Volume 2 Display Name')
        fake_volume2_dict = meta.obj_to_munch(fake_volume2)
        self.register_uris([
            self.get_cinder_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', 'detail']),
                 json={'volumes': [fake_volume_dict]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', 'detail']),
                 json={'volumes': [fake_volume_dict, fake_volume2_dict]})])
        for a, b in zip([fake_volume_dict],
                        self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        for a, b in zip([fake_volume_dict, fake_volume2_dict],
                        self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        self.assert_calls()

    def test_create_volume_invalidates(self):
        fake_volb4 = meta.obj_to_munch(
            fakes.FakeVolume('volume1', 'available', ''))
        _id = '12345'
        fake_vol_creating = meta.obj_to_munch(
            fakes.FakeVolume(_id, 'creating', ''))
        fake_vol_avail = meta.obj_to_munch(
            fakes.FakeVolume(_id, 'available', ''))

        def now_deleting(request, context):
            fake_vol_avail['status'] = 'deleting'

        self.register_uris([
            self.get_cinder_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', 'detail']),
                 json={'volumes': [fake_volb4]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes']),
                 json={'volume': fake_vol_creating}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', _id]),
                 json={'volume': fake_vol_creating}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', _id]),
                 json={'volume': fake_vol_avail}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', 'detail']),
                 json={'volumes': [fake_volb4, fake_vol_avail]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['volumes', _id]),
                 json={'volume': fake_vol_avail}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', _id]),
                 json=now_deleting),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', _id]),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['volumes', 'detail']),
                 json={'volumes': [fake_volb4, fake_vol_avail]}),
        ])

        for a, b in zip([fake_volb4], self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        volume = dict(display_name='junk_vol',
                      size=1,
                      display_description='test junk volume')
        self.cloud.create_volume(wait=True, timeout=2, **volume)
        # If cache was not invalidated, we would not see our own volume here
        # because the first volume was available and thus would already be
        # cached.
        for a, b in zip([fake_volb4, fake_vol_avail],
                        self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        self.cloud.delete_volume(_id)
        # And now delete and check same thing since list is cached as all
        # available
        for a, b in zip([fake_volb4], self.cloud.list_volumes()):
            self._compare_volumes(a, b)
        self.assert_calls()

    def test_list_users(self):
        user_data = self._get_user_data(email='test@example.com')
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     service_type='identity',
                     resource='users',
                     base_url_append='v3'),
                 status_code=200,
                 json={'users': [user_data.json_response['user']]})])
        users = self.cloud.list_users()
        self.assertEqual(1, len(users))
        self.assertEqual(user_data.user_id, users[0]['id'])
        self.assertEqual(user_data.name, users[0]['name'])
        self.assertEqual(user_data.email, users[0]['email'])
        self.assert_calls()

    def test_modify_user_invalidates_cache(self):
        self.use_keystone_v2()

        user_data = self._get_user_data(email='test@example.com')
        new_resp = {'user': user_data.json_response['user'].copy()}
        new_resp['user']['email'] = 'Nope@Nope.Nope'
        new_req = {'user': {'email': new_resp['user']['email']}}

        mock_users_url = self.get_mock_url(
            service_type='identity',
            interface='admin',
            resource='users')
        mock_user_resource_url = self.get_mock_url(
            service_type='identity',
            interface='admin',
            resource='users',
            append=[user_data.user_id])

        empty_user_list_resp = {'users': []}
        users_list_resp = {'users': [user_data.json_response['user']]}
        updated_users_list_resp = {'users': [new_resp['user']]}

        # Password is None in the original create below
        del user_data.json_request['user']['password']

        uris_to_mock = [
            # Inital User List is Empty
            dict(method='GET', uri=mock_users_url, status_code=200,
                 json=empty_user_list_resp),
            # POST to create the user
            # GET to get the user data after POST
            dict(method='POST', uri=mock_users_url, status_code=200,
                 json=user_data.json_response,
                 validate=dict(json=user_data.json_request)),
            # List Users Call
            dict(method='GET', uri=mock_users_url, status_code=200,
                 json=users_list_resp),
            # List users to get ID for update
            # Get user using user_id from list
            # Update user
            # Get updated user
            dict(method='GET', uri=mock_users_url, status_code=200,
                 json=users_list_resp),
            dict(method='PUT', uri=mock_user_resource_url, status_code=200,
                 json=new_resp, validate=dict(json=new_req)),
            # List Users Call
            dict(method='GET', uri=mock_users_url, status_code=200,
                 json=updated_users_list_resp),
            # List User to get ID for delete
            # delete user
            dict(method='GET', uri=mock_users_url, status_code=200,
                 json=updated_users_list_resp),
            dict(method='DELETE', uri=mock_user_resource_url, status_code=204),
            # List Users Call (empty post delete)
            dict(method='GET', uri=mock_users_url, status_code=200,
                 json=empty_user_list_resp)
        ]

        self.register_uris(uris_to_mock)

        # first cache an empty list
        self.assertEqual([], self.cloud.list_users())

        # now add one
        created = self.cloud.create_user(name=user_data.name,
                                         email=user_data.email)
        self.assertEqual(user_data.user_id, created['id'])
        self.assertEqual(user_data.name, created['name'])
        self.assertEqual(user_data.email, created['email'])
        # Cache should have been invalidated
        users = self.cloud.list_users()
        self.assertEqual(user_data.user_id, users[0]['id'])
        self.assertEqual(user_data.name, users[0]['name'])
        self.assertEqual(user_data.email, users[0]['email'])

        # Update and check to see if it is updated
        updated = self.cloud.update_user(user_data.user_id,
                                         email=new_resp['user']['email'])
        self.assertEqual(user_data.user_id, updated.id)
        self.assertEqual(user_data.name, updated.name)
        self.assertEqual(new_resp['user']['email'], updated.email)
        users = self.cloud.list_users()
        self.assertEqual(1, len(users))
        self.assertEqual(user_data.user_id, users[0]['id'])
        self.assertEqual(user_data.name, users[0]['name'])
        self.assertEqual(new_resp['user']['email'], users[0]['email'])
        # Now delete and ensure it disappears
        self.cloud.delete_user(user_data.user_id)
        self.assertEqual([], self.cloud.list_users())
        self.assert_calls()

    def test_list_flavors(self):
        mock_uri = '{endpoint}/flavors/detail?is_public=None'.format(
            endpoint=fakes.COMPUTE_ENDPOINT)

        uris_to_mock = [
            dict(method='GET', uri=mock_uri,
                 validate=dict(
                     headers={'OpenStack-API-Version': 'compute 2.53'}),
                 json={'flavors': []}),
            dict(method='GET', uri=mock_uri,
                 validate=dict(
                     headers={'OpenStack-API-Version': 'compute 2.53'}),
                 json={'flavors': fakes.FAKE_FLAVOR_LIST})
        ]
        self.use_compute_discovery()

        self.register_uris(uris_to_mock)

        self.assertEqual([], self.cloud.list_flavors())

        self.assertEqual([], self.cloud.list_flavors())

        self.cloud.list_flavors.invalidate(self.cloud)
        self.assertResourceListEqual(
            self.cloud.list_flavors(),
            fakes.FAKE_FLAVOR_LIST,
            _flavor.Flavor
        )

        self.assert_calls()

    def test_list_images(self):

        self.use_glance()
        fake_image = fakes.make_fake_image(image_id='42')

        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url('image', 'public',
                                       append=['v2', 'images']),
                 json={'images': []}),
            dict(method='GET',
                 uri=self.get_mock_url('image', 'public',
                                       append=['v2', 'images']),
                 json={'images': [fake_image]}),
        ])

        self.assertEqual([], self.cloud.list_images())
        self.assertEqual([], self.cloud.list_images())
        self.cloud.list_images.invalidate(self.cloud)
        [self._compare_images(a, b) for a, b in zip(
            [fake_image],
            self.cloud.list_images())]

        self.assert_calls()

    def test_list_images_caches_deleted_status(self):
        self.use_glance()

        deleted_image_id = self.getUniqueString()
        deleted_image = fakes.make_fake_image(
            image_id=deleted_image_id, status='deleted')
        active_image_id = self.getUniqueString()
        active_image = fakes.make_fake_image(image_id=active_image_id)
        list_return = {'images': [active_image, deleted_image]}
        self.register_uris([
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=list_return),
        ])

        [self._compare_images(a, b) for a, b in zip(
            [active_image],
            self.cloud.list_images())]

        [self._compare_images(a, b) for a, b in zip(
            [active_image],
            self.cloud.list_images())]

        # We should only have one call
        self.assert_calls()

    def test_cache_no_cloud_name(self):
        self.use_glance()

        self.cloud.name = None
        fi = fakes.make_fake_image(image_id=self.getUniqueString())
        fi2 = fakes.make_fake_image(image_id=self.getUniqueString())

        self.register_uris([
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json={'images': [fi]}),
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json={'images': [fi, fi2]}),
        ])

        [self._compare_images(a, b) for a, b in zip(
            [fi],
            self.cloud.list_images())]

        # Now test that the list was cached
        [self._compare_images(a, b) for a, b in zip(
            [fi],
            self.cloud.list_images())]

        # Invalidation too
        self.cloud.list_images.invalidate(self.cloud)
        [self._compare_images(a, b) for a, b in zip(
            [fi, fi2],
            self.cloud.list_images())]

    def test_list_ports_filtered(self):
        down_port = test_port.TestPort.mock_neutron_port_create_rep['port']
        active_port = down_port.copy()
        active_port['status'] = 'ACTIVE'
        # We're testing to make sure a query string is passed when we're
        # caching (cache by url), and that the results are still filtered.
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'ports'],
                     qs_elements=['status=DOWN']),
                 json={'ports': [
                     down_port,
                     active_port,
                 ]}),
        ])
        ports = self.cloud.list_ports(filters={'status': 'DOWN'})
        for a, b in zip([down_port], ports):
            self.assertDictEqual(
                _port.Port(**a).to_dict(computed=False),
                b.to_dict(computed=False))
        self.assert_calls()


class TestCacheIgnoresQueuedStatus(base.TestCase):

    scenarios = [
        ('queued', dict(status='queued')),
        ('saving', dict(status='saving')),
        ('pending_delete', dict(status='pending_delete')),
    ]

    def setUp(self):
        super(TestCacheIgnoresQueuedStatus, self).setUp(
            cloud_config_fixture='clouds_cache.yaml')
        self.use_glance()
        active_image_id = self.getUniqueString()
        self.active_image = fakes.make_fake_image(
            image_id=active_image_id, status=self.status)
        self.active_list_return = {'images': [self.active_image]}
        steady_image_id = self.getUniqueString()
        self.steady_image = fakes.make_fake_image(image_id=steady_image_id)
        self.steady_list_return = {
            'images': [self.active_image, self.steady_image]}

    def _compare_images(self, exp, real):
        self.assertDictEqual(
            _image.Image(**exp).to_dict(computed=False),
            real.to_dict(computed=False))

    def test_list_images_ignores_pending_status(self):

        self.register_uris([
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=self.active_list_return),
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=self.steady_list_return),
        ])

        [self._compare_images(a, b) for a, b in zip(
            [self.active_image],
            self.cloud.list_images())]

        # Should expect steady_image to appear if active wasn't cached
        [self._compare_images(a, b) for a, b in zip(
            [self.active_image, self.steady_image],
            self.cloud.list_images())]


class TestCacheSteadyStatus(base.TestCase):

    scenarios = [
        ('active', dict(status='active')),
        ('killed', dict(status='killed')),
    ]

    def setUp(self):
        super(TestCacheSteadyStatus, self).setUp(
            cloud_config_fixture='clouds_cache.yaml')
        self.use_glance()
        active_image_id = self.getUniqueString()
        self.active_image = fakes.make_fake_image(
            image_id=active_image_id, status=self.status)
        self.active_list_return = {'images': [self.active_image]}

    def _compare_images(self, exp, real):
        self.assertDictEqual(
            _image.Image(**exp).to_dict(computed=False),
            real.to_dict(computed=False))

    def test_list_images_caches_steady_status(self):

        self.register_uris([
            dict(method='GET',
                 uri='https://image.example.com/v2/images',
                 json=self.active_list_return),
        ])

        [self._compare_images(a, b) for a, b in zip(
            [self.active_image],
            self.cloud.list_images())]

        [self._compare_images(a, b) for a, b in zip(
            [self.active_image],
            self.cloud.list_images())]

        # We should only have one call
        self.assert_calls()


class TestBogusAuth(base.TestCase):

    def setUp(self):
        super(TestBogusAuth, self).setUp(
            cloud_config_fixture='clouds_cache.yaml')

    def test_get_auth_bogus(self):
        with testtools.ExpectedException(exceptions.ConfigException):
            openstack.connect(
                cloud='_bogus_test_', config=self.config)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import vpn_ikepolicy
from openstack.tests.unit import base


EXAMPLE = {
    "auth_algorithm": "1",
    "description": "2",
    "encryption_algorithm": "3",
    "ike_version": "4",
    "lifetime": {'a': 5},
    "name": "5",
    "pfs": "6",
    "project_id": "7",
    "phase1_negotiation_mode": "8",
    "units": "9",
    "value": 10
}


class TestVpnIkePolicy(base.TestCase):

    def test_basic(self):
        sot = vpn_ikepolicy.VpnIkePolicy()
        self.assertEqual('ikepolicy', sot.resource_key)
        self.assertEqual('ikepolicies', sot.resources_key)
        self.assertEqual('/vpn/ikepolicies', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = vpn_ikepolicy.VpnIkePolicy(**EXAMPLE)
        self.assertEqual(EXAMPLE['auth_algorithm'], sot.auth_algorithm)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['encryption_algorithm'],
                         sot.encryption_algorithm)
        self.assertEqual(EXAMPLE['ike_version'], sot.ike_version)
        self.assertEqual(EXAMPLE['lifetime'], sot.lifetime)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['pfs'], sot.pfs)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['phase1_negotiation_mode'],
                         sot.phase1_negotiation_mode)
        self.assertEqual(EXAMPLE['units'], sot.units)
        self.assertEqual(EXAMPLE['value'], sot.value)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# import types so that we can reference ListType in sphinx param declarations.
# We can't just use list, because sphinx gets confused by
# openstack.resource.Resource.list and openstack.resource2.Resource.list
import types  # noqa


class ClusteringCloudMixin:
    pass

    @property
    def _clustering_client(self):
        if 'clustering' not in self._raw_clients:
            clustering_client = self._get_versioned_client(
                'clustering', min_version=1, max_version='1.latest')
            self._raw_clients['clustering'] = clustering_client
        return self._raw_clients['clustering']

# NOTE(gtema): work on getting rid of direct API calls showed that this
# implementation never worked properly and tests in reality verifying wrong
# things. Unless someone is really interested in this piece of code this will
# be commented out and apparently dropped completely. This has no impact on the
# proxy layer.

#    def create_cluster(self, name, profile, config=None, desired_capacity=0,
#                       max_size=None, metadata=None, min_size=None,
#                       timeout=None):
#        profile = self.get_cluster_profile(profile)
#        profile_id = profile['id']
#        body = {
#            'desired_capacity': desired_capacity,
#            'name': name,
#            'profile_id': profile_id
#        }
#
#        if config is not None:
#            body['config'] = config
#
#        if max_size is not None:
#            body['max_size'] = max_size
#
#        if metadata is not None:
#            body['metadata'] = metadata
#
#        if min_size is not None:
#            body['min_size'] = min_size
#
#        if timeout is not None:
#            body['timeout'] = timeout
#
#        return self.clustering.create_cluster(**body)
#
#    def set_cluster_metadata(self, name_or_id, metadata):
#        cluster = self.get_cluster(name_or_id)
#        if not cluster:
#            raise exc.OpenStackCloudException(
#                'Invalid Cluster {cluster}'.format(cluster=name_or_id))
#        self.clustering.set_cluster_metadata(cluster, **metadata)
#
#    def get_cluster_by_id(self, cluster_id):
#        try:
#            return self.get_cluster(cluster_id)
#        except Exception:
#            return None
#
#    def get_cluster(self, name_or_id, filters=None):
#        return _utils._get_entity(
#            cloud=self, resource='cluster',
#            name_or_id=name_or_id, filters=filters)
#
#    def update_cluster(self, name_or_id, new_name=None,
#                       profile_name_or_id=None, config=None, metadata=None,
#                       timeout=None, profile_only=False):
#        old_cluster = self.get_cluster(name_or_id)
#        if old_cluster is None:
#            raise exc.OpenStackCloudException(
#                'Invalid Cluster {cluster}'.format(cluster=name_or_id))
#        cluster = {
#            'profile_only': profile_only
#        }
#
#        if config is not None:
#            cluster['config'] = config
#
#        if metadata is not None:
#            cluster['metadata'] = metadata
#
#        if profile_name_or_id is not None:
#            profile = self.get_cluster_profile(profile_name_or_id)
#            if profile is None:
#                raise exc.OpenStackCloudException(
#                    'Invalid Cluster Profile {profile}'.format(
#                        profile=profile_name_or_id))
#            cluster['profile_id'] = profile.id
#
#        if timeout is not None:
#            cluster['timeout'] = timeout
#
#        if new_name is not None:
#            cluster['name'] = new_name
#
#        return self.update_cluster(old_cluster, cluster)
#
#    def delete_cluster(self, name_or_id):
#        cluster = self.get_cluster(name_or_id)
#        if cluster is None:
#            self.log.debug("Cluster %s not found for deleting", name_or_id)
#            return False
#
#        for policy in self.list_policies_on_cluster(name_or_id):
#            detach_policy = self.get_cluster_policy_by_id(
#                policy['policy_id'])
#            self.detach_policy_from_cluster(cluster, detach_policy)
#
#        for receiver in self.list_cluster_receivers():
#            if cluster["id"] == receiver["cluster_id"]:
#                self.delete_cluster_receiver(receiver["id"], wait=True)
#
#        self.clustering.delete_cluster(cluster)
#
#        return True
#
#    def search_clusters(self, name_or_id=None, filters=None):
#        clusters = self.list_clusters()
#        return _utils._filter_list(clusters, name_or_id, filters)
#
#    def list_clusters(self):
#        return list(self.clustering.clusters())
#
#    def attach_policy_to_cluster(
#        self, name_or_id, policy_name_or_id, is_enabled
#    ):
#        cluster = self.get_cluster(name_or_id)
#        policy = self.get_cluster_policy(policy_name_or_id)
#        if cluster is None:
#            raise exc.OpenStackCloudException(
#                'Cluster {cluster} not found for attaching'.format(
#                    cluster=name_or_id))
#
#        if policy is None:
#            raise exc.OpenStackCloudException(
#                'Policy {policy} not found for attaching'.format(
#                    policy=policy_name_or_id))
#
#        self.clustering.attach_policy_to_cluster(cluster, policy)
#
#        return True
#
#    def detach_policy_from_cluster(
#            self, name_or_id, policy_name_or_id, wait=False, timeout=3600):
#        cluster = self.get_cluster(name_or_id)
#        policy = self.get_cluster_policy(policy_name_or_id)
#        if cluster is None:
#            raise exc.OpenStackCloudException(
#                'Cluster {cluster} not found for detaching'.format(
#                    cluster=name_or_id))
#
#        if policy is None:
#            raise exc.OpenStackCloudException(
#                'Policy {policy} not found for detaching'.format(
#                    policy=policy_name_or_id))
#
#        self.clustering.detach_policy_from_cluster(cluster, policy)
#
#        if not wait:
#            return True
#
#        for count in utils.iterate_timeout(
#                timeout, "Timeout waiting for cluster policy to detach"):
#
#            policies = self.get_cluster_by_id(cluster['id']).policies
#
#            # NOTE(gtema): we iterate over all current policies and "continue"
#            # if selected policy is still there
#            is_present = False
#            for active_policy in policies:
#                if active_policy.id == policy.id:
#                    is_present = True
#            if not is_present:
#                break
#        return True
#
#    def update_policy_on_cluster(self, name_or_id, policy_name_or_id,
#                                 is_enabled):
#        cluster = self.get_cluster(name_or_id)
#        policy = self.get_cluster_policy(policy_name_or_id)
#        if cluster is None:
#            raise exc.OpenStackCloudException(
#                'Cluster {cluster} not found for updating'.format(
#                    cluster=name_or_id))
#
#        if policy is None:
#            raise exc.OpenStackCloudException(
#                'Policy {policy} not found for updating'.format(
#                    policy=policy_name_or_id))
#
#        self.clustering.update_cluster_policy(
#            cluster, policy, is_enabled=is_enabled)
#
#        return True
#
#    def get_policy_on_cluster(self, name_or_id, policy_name_or_id):
#        cluster = self.get_cluster(name_or_id)
#        policy = self.get_policy(policy_name_or_id)
#        if policy and cluster:
#            return self.clustering.get_cluster_policy(
#            policy, cluster)
#        else:
#            return False
#
#    def list_policies_on_cluster(self, name_or_id):
#        cluster = self.get_cluster(name_or_id)
#        return list(self.clustering.cluster_policies(cluster))
#
#    def create_cluster_profile(self, name, spec, metadata=None):
#        profile = {
#            'name': name,
#            'spec': spec
#        }
#
#        if metadata is not None:
#            profile['metadata'] = metadata
#
#        data = self._clustering_client.post(
#            '/profiles', json={'profile': profile},
#            error_message="Error creating profile {name}".format(name=name))
#
#        return self._get_and_munchify('profile', data)
#
#    def set_cluster_profile_metadata(self, name_or_id, metadata):
#        profile = self.get_cluster_profile(name_or_id)
#        if not profile:
#            raise exc.OpenStackCloudException(
#                'Invalid Profile {profile}'.format(profile=name_or_id))
#
#        self._clustering_client.post(
#            '/profiles/{profile_id}/metadata'.format(profile_id=profile['id']),
#            json={'metadata': metadata},
#            error_message='Error updating profile metadata')
#
#    def search_cluster_profiles(self, name_or_id=None, filters=None):
#        cluster_profiles = self.list_cluster_profiles()
#        return _utils._filter_list(cluster_profiles, name_or_id, filters)
#
#    def list_cluster_profiles(self):
#        try:
#            data = self._clustering_client.get(
#                '/profiles',
#                error_message="Error fetching profiles")
#        except exc.OpenStackCloudURINotFound as e:
#            self.log.debug(str(e), exc_info=True)
#            return []
#        return self._get_and_munchify('profiles', data)
#
#    def get_cluster_profile_by_id(self, profile_id):
#        try:
#            data = self._clustering_client.get(
#                "/profiles/{profile_id}".format(profile_id=profile_id),
#                error_message="Error fetching profile {name}".format(
#                    name=profile_id))
#            return self._get_and_munchify('profile', data)
#        except exc.OpenStackCloudURINotFound as e:
#            self.log.debug(str(e), exc_info=True)
#            return None
#
#    def get_cluster_profile(self, name_or_id, filters=None):
#        return _utils._get_entity(self, 'cluster_profile', name_or_id,
#                                  filters)
#
#    def delete_cluster_profile(self, name_or_id):
#        profile = self.get_cluster_profile(name_or_id)
#        if profile is None:
#            self.log.debug("Profile %s not found for deleting", name_or_id)
#            return False
#
#        for cluster in self.list_clusters():
#            if (name_or_id, profile.id) in cluster.items():
#                self.log.debug(
#                    "Profile %s is being used by cluster %s, won't delete",
#                    name_or_id, cluster.name)
#                return False
#
#        self._clustering_client.delete(
#            "/profiles/{profile_id}".format(profile_id=profile['id']),
#            error_message="Error deleting profile "
#                          "{name}".format(name=name_or_id))
#
#        return True
#
#    def update_cluster_profile(
#        self, name_or_id, metadata=None, new_name=None
#    ):
#        old_profile = self.get_cluster_profile(name_or_id)
#        if not old_profile:
#            raise exc.OpenStackCloudException(
#                'Invalid Profile {profile}'.format(profile=name_or_id))
#
#        profile = {}
#
#        if metadata is not None:
#            profile['metadata'] = metadata
#
#        if new_name is not None:
#            profile['name'] = new_name
#
#        data = self._clustering_client.patch(
#            "/profiles/{profile_id}".format(profile_id=old_profile.id),
#            json={'profile': profile},
#            error_message="Error updating profile {name}".format(
#                name=name_or_id))
#
#        return self._get_and_munchify(key=None, data=data)
#
#    def create_cluster_policy(self, name, spec):
#        policy = {
#            'name': name,
#            'spec': spec
#        }
#
#        data = self._clustering_client.post(
#            '/policies', json={'policy': policy},
#            error_message="Error creating policy {name}".format(
#                name=policy['name']))
#        return self._get_and_munchify('policy', data)
#
#    def search_cluster_policies(self, name_or_id=None, filters=None):
#        cluster_policies = self.list_cluster_policies()
#        return _utils._filter_list(cluster_policies, name_or_id, filters)
#
#    def list_cluster_policies(self):
#        endpoint = "/policies"
#        try:
#            data = self._clustering_client.get(
#                endpoint,
#                error_message="Error fetching cluster policies")
#        except exc.OpenStackCloudURINotFound as e:
#            self.log.debug(str(e), exc_info=True)
#            return []
#        return self._get_and_munchify('policies', data)
#
#    def get_cluster_policy_by_id(self, policy_id):
#        try:
#            data = self._clustering_client.get(
#                "/policies/{policy_id}".format(policy_id=policy_id),
#                error_message="Error fetching policy {name}".format(
#                    name=policy_id))
#            return self._get_and_munchify('policy', data)
#        except exc.OpenStackCloudURINotFound as e:
#            self.log.debug(str(e), exc_info=True)
#            return None
#
#    def get_cluster_policy(self, name_or_id, filters=None):
#        return _utils._get_entity(
#            self, 'cluster_policie', name_or_id, filters)
#
#    def delete_cluster_policy(self, name_or_id):
#        policy = self.get_cluster_policy_by_id(name_or_id)
#        if policy is None:
#            self.log.debug("Policy %s not found for deleting", name_or_id)
#            return False
#
#        for cluster in self.list_clusters():
#            if (name_or_id, policy.id) in cluster.items():
#                self.log.debug(
#                    "Policy %s is being used by cluster %s, won't delete",
#                    name_or_id, cluster.name)
#                return False
#
#        self._clustering_client.delete(
#            "/policies/{policy_id}".format(policy_id=name_or_id),
#            error_message="Error deleting policy "
#                          "{name}".format(name=name_or_id))
#
#        return True
#
#    def update_cluster_policy(self, name_or_id, new_name):
#        old_policy = self.get_cluster_policy(name_or_id)
#        if not old_policy:
#            raise exc.OpenStackCloudException(
#                'Invalid Policy {policy}'.format(policy=name_or_id))
#        policy = {'name': new_name}
#
#        data = self._clustering_client.patch(
#            "/policies/{policy_id}".format(policy_id=old_policy.id),
#            json={'policy': policy},
#            error_message="Error updating policy "
#                          "{name}".format(name=name_or_id))
#        return self._get_and_munchify(key=None, data=data)
#
#    def create_cluster_receiver(self, name, receiver_type,
#                                cluster_name_or_id=None, action=None,
#                                actor=None, params=None):
#        cluster = self.get_cluster(cluster_name_or_id)
#        if cluster is None:
#            raise exc.OpenStackCloudException(
#                'Invalid cluster {cluster}'.format(
#                    cluster=cluster_name_or_id))
#
#        receiver = {
#            'name': name,
#            'type': receiver_type
#        }
#
#        if cluster_name_or_id is not None:
#            receiver['cluster_id'] = cluster.id
#
#        if action is not None:
#            receiver['action'] = action
#
#        if actor is not None:
#            receiver['actor'] = actor
#
#        if params is not None:
#            receiver['params'] = params
#
#        data = self._clustering_client.post(
#            '/receivers', json={'receiver': receiver},
#            error_message="Error creating receiver {name}".format(name=name))
#        return self._get_and_munchify('receiver', data)
#
#    def search_cluster_receivers(self, name_or_id=None, filters=None):
#        cluster_receivers = self.list_cluster_receivers()
#        return _utils._filter_list(cluster_receivers, name_or_id, filters)
#
#    def list_cluster_receivers(self):
#        try:
#            data = self._clustering_client.get(
#                '/receivers',
#                error_message="Error fetching receivers")
#        except exc.OpenStackCloudURINotFound as e:
#            self.log.debug(str(e), exc_info=True)
#            return []
#        return self._get_and_munchify('receivers', data)
#
#    def get_cluster_receiver_by_id(self, receiver_id):
#        try:
#            data = self._clustering_client.get(
#                "/receivers/{receiver_id}".format(receiver_id=receiver_id),
#                error_message="Error fetching receiver {name}".format(
#                    name=receiver_id))
#            return self._get_and_munchify('receiver', data)
#        except exc.OpenStackCloudURINotFound as e:
#            self.log.debug(str(e), exc_info=True)
#            return None
#
#    def get_cluster_receiver(self, name_or_id, filters=None):
#        return _utils._get_entity(
#            self, 'cluster_receiver', name_or_id, filters)
#
#    def delete_cluster_receiver(self, name_or_id, wait=False, timeout=3600):
#        receiver = self.get_cluster_receiver(name_or_id)
#        if receiver is None:
#            self.log.debug("Receiver %s not found for deleting", name_or_id)
#            return False
#
#        receiver_id = receiver['id']
#
#        self._clustering_client.delete(
#            "/receivers/{receiver_id}".format(receiver_id=receiver_id),
#            error_message="Error deleting receiver {name}".format(
#                name=name_or_id))
#
#        if not wait:
#            return True
#
#        for count in utils.iterate_timeout(
#                timeout, "Timeout waiting for cluster receiver to delete"):
#
#            receiver = self.get_cluster_receiver_by_id(receiver_id)
#
#            if not receiver:
#                break
#
#        return True
#
#    def update_cluster_receiver(self, name_or_id, new_name=None, action=None,
#                                params=None):
#        old_receiver = self.get_cluster_receiver(name_or_id)
#        if old_receiver is None:
#            raise exc.OpenStackCloudException(
#                'Invalid receiver {receiver}'.format(receiver=name_or_id))
#
#        receiver = {}
#
#        if new_name is not None:
#            receiver['name'] = new_name
#
#        if action is not None:
#            receiver['action'] = action
#
#        if params is not None:
#            receiver['params'] = params
#
#        data = self._clustering_client.patch(
#            "/receivers/{receiver_id}".format(receiver_id=old_receiver.id),
#            json={'receiver': receiver},
#            error_message="Error updating receiver {name}".format(
#                name=name_or_id))
#        return self._get_and_munchify(key=None, data=data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import openstack.cloud
from openstack.tests import fakes
from openstack.tests.unit import base


class TestFlavors(base.TestCase):

    def setUp(self):
        super(TestFlavors, self).setUp()
        # self.use_compute_discovery()

    def test_create_flavor(self):

        self.use_compute_discovery()
        self.register_uris([
            dict(method='POST',
                 uri='{endpoint}/flavors'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavor': fakes.FAKE_FLAVOR},
                 validate=dict(
                     json={
                         'flavor': {
                             "name": "vanilla",
                             "ram": 65536,
                             "vcpus": 24,
                             "swap": 0,
                             "os-flavor-access:is_public": True,
                             "rxtx_factor": 1.0,
                             "OS-FLV-EXT-DATA:ephemeral": 0,
                             "disk": 1600,
                             "id": None}}))])

        self.cloud.create_flavor(
            'vanilla', ram=65536, disk=1600, vcpus=24,
        )
        self.assert_calls()

    def test_delete_flavor(self):
        self.use_compute_discovery()
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/flavors/vanilla'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=fakes.FAKE_FLAVOR),
            dict(method='DELETE',
                 uri='{endpoint}/flavors/{id}'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id=fakes.FLAVOR_ID))])
        self.assertTrue(self.cloud.delete_flavor('vanilla'))

        self.assert_calls()

    def test_delete_flavor_not_found(self):
        self.use_compute_discovery()
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/flavors/invalid'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 status_code=404),
            dict(method='GET',
                 uri='{endpoint}/flavors/detail?is_public=None'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavors': fakes.FAKE_FLAVOR_LIST})])

        self.assertFalse(self.cloud.delete_flavor('invalid'))

        self.assert_calls()

    def test_delete_flavor_exception(self):
        self.use_compute_discovery()
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/flavors/vanilla'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=fakes.FAKE_FLAVOR),
            dict(method='GET',
                 uri='{endpoint}/flavors/detail?is_public=None'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavors': fakes.FAKE_FLAVOR_LIST}),
            dict(method='DELETE',
                 uri='{endpoint}/flavors/{id}'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id=fakes.FLAVOR_ID),
                 status_code=503)])

        self.assertRaises(openstack.cloud.OpenStackCloudException,
                          self.cloud.delete_flavor, 'vanilla')

    def test_list_flavors(self):
        self.use_compute_discovery()
        uris_to_mock = [
            dict(method='GET',
                 uri='{endpoint}/flavors/detail?is_public=None'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavors': fakes.FAKE_FLAVOR_LIST}),
        ]
        self.register_uris(uris_to_mock)

        flavors = self.cloud.list_flavors()

        # test that new flavor is created correctly
        found = False
        for flavor in flavors:
            if flavor['name'] == 'vanilla':
                found = True
                break
        self.assertTrue(found)
        needed_keys = {'name', 'ram', 'vcpus', 'id', 'is_public', 'disk'}
        if found:
            # check flavor content
            self.assertTrue(needed_keys.issubset(flavor.keys()))
        self.assert_calls()

    def test_list_flavors_with_extra(self):
        self.use_compute_discovery()
        uris_to_mock = [
            dict(method='GET',
                 uri='{endpoint}/flavors/detail?is_public=None'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavors': fakes.FAKE_FLAVOR_LIST}),
        ]
        uris_to_mock.extend([
            dict(method='GET',
                 uri='{endpoint}/flavors/{id}/os-extra_specs'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id=flavor['id']),
                 json={'extra_specs': {}})
            for flavor in fakes.FAKE_FLAVOR_LIST])
        self.register_uris(uris_to_mock)

        flavors = self.cloud.list_flavors(get_extra=True)

        # test that new flavor is created correctly
        found = False
        for flavor in flavors:
            if flavor['name'] == 'vanilla':
                found = True
                break
        self.assertTrue(found)
        needed_keys = {'name', 'ram', 'vcpus', 'id', 'is_public', 'disk'}
        if found:
            # check flavor content
            self.assertTrue(needed_keys.issubset(flavor.keys()))
        self.assert_calls()

    def test_get_flavor_by_ram(self):
        self.use_compute_discovery()
        uris_to_mock = [
            dict(method='GET',
                 uri='{endpoint}/flavors/detail?is_public=None'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavors': fakes.FAKE_FLAVOR_LIST}),
        ]
        uris_to_mock.extend([
            dict(method='GET',
                 uri='{endpoint}/flavors/{id}/os-extra_specs'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id=flavor['id']),
                 json={'extra_specs': {}})
            for flavor in fakes.FAKE_FLAVOR_LIST])
        self.register_uris(uris_to_mock)

        flavor = self.cloud.get_flavor_by_ram(ram=250)
        self.assertEqual(fakes.STRAWBERRY_FLAVOR_ID, flavor['id'])

    def test_get_flavor_by_ram_and_include(self):
        self.use_compute_discovery()
        uris_to_mock = [
            dict(method='GET',
                 uri='{endpoint}/flavors/detail?is_public=None'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavors': fakes.FAKE_FLAVOR_LIST}),
        ]
        uris_to_mock.extend([
            dict(method='GET',
                 uri='{endpoint}/flavors/{id}/os-extra_specs'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id=flavor['id']),
                 json={'extra_specs': {}})
            for flavor in fakes.FAKE_FLAVOR_LIST])
        self.register_uris(uris_to_mock)
        flavor = self.cloud.get_flavor_by_ram(ram=150, include='strawberry')
        self.assertEqual(fakes.STRAWBERRY_FLAVOR_ID, flavor['id'])

    def test_get_flavor_by_ram_not_found(self):
        self.use_compute_discovery()
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/flavors/detail?is_public=None'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'flavors': []})])
        self.assertRaises(
            openstack.cloud.OpenStackCloudException,
            self.cloud.get_flavor_by_ram,
            ram=100)

    def test_get_flavor_string_and_int(self):
        self.use_compute_discovery()
        flavor_resource_uri = '{endpoint}/flavors/1/os-extra_specs'.format(
            endpoint=fakes.COMPUTE_ENDPOINT)
        flavor = fakes.make_fake_flavor('1', 'vanilla')
        flavor_json = {'extra_specs': {}}

        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/flavors/1'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=flavor),
            dict(method='GET', uri=flavor_resource_uri, json=flavor_json),
        ])

        flavor1 = self.cloud.get_flavor('1')
        self.assertEqual('1', flavor1['id'])
        flavor2 = self.cloud.get_flavor(1)
        self.assertEqual('1', flavor2['id'])

    def test_set_flavor_specs(self):
        self.use_compute_discovery()
        extra_specs = dict(key1='value1')
        self.register_uris([
            dict(method='POST',
                 uri='{endpoint}/flavors/{id}/os-extra_specs'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id=1),
                 json=dict(extra_specs=extra_specs))])

        self.cloud.set_flavor_specs(1, extra_specs)
        self.assert_calls()

    def test_unset_flavor_specs(self):
        self.use_compute_discovery()
        keys = ['key1', 'key2']
        self.register_uris([
            dict(method='DELETE',
                 uri='{endpoint}/flavors/{id}/os-extra_specs/{key}'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id=1, key=key))
            for key in keys])

        self.cloud.unset_flavor_specs(1, keys)
        self.assert_calls()

    def test_add_flavor_access(self):
        self.register_uris([
            dict(method='POST',
                 uri='{endpoint}/flavors/{id}/action'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id='flavor_id'),
                 json={
                     'flavor_access': [{
                         'flavor_id': 'flavor_id', 'tenant_id': 'tenant_id'}]},
                 validate=dict(
                     json={'addTenantAccess': {'tenant': 'tenant_id'}}))])

        self.cloud.add_flavor_access('flavor_id', 'tenant_id')
        self.assert_calls()

    def test_remove_flavor_access(self):
        self.register_uris([
            dict(method='POST',
                 uri='{endpoint}/flavors/{id}/action'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT, id='flavor_id'),
                 json={'flavor_access': []},
                 validate=dict(
                     json={'removeTenantAccess': {'tenant': 'tenant_id'}}))])

        self.cloud.remove_flavor_access('flavor_id', 'tenant_id')
        self.assert_calls()

    def test_list_flavor_access(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/flavors/vanilla/os-flavor-access'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={
                     'flavor_access': [
                         {'flavor_id': 'vanilla', 'tenant_id': 'tenant_id'}]})
        ])
        self.cloud.list_flavor_access('vanilla')
        self.assert_calls()

    def test_get_flavor_by_id(self):
        self.use_compute_discovery()
        flavor_uri = '{endpoint}/flavors/1'.format(
            endpoint=fakes.COMPUTE_ENDPOINT)
        flavor_json = {'flavor': fakes.make_fake_flavor('1', 'vanilla')}

        self.register_uris([
            dict(method='GET', uri=flavor_uri, json=flavor_json),
        ])

        flavor1 = self.cloud.get_flavor_by_id('1')
        self.assertEqual('1', flavor1['id'])
        self.assertEqual({}, flavor1.extra_specs)
        flavor2 = self.cloud.get_flavor_by_id('1')
        self.assertEqual('1', flavor2['id'])
        self.assertEqual({}, flavor2.extra_specs)

    def test_get_flavor_with_extra_specs(self):
        self.use_compute_discovery()
        flavor_uri = '{endpoint}/flavors/1'.format(
            endpoint=fakes.COMPUTE_ENDPOINT)
        flavor_extra_uri = '{endpoint}/flavors/1/os-extra_specs'.format(
            endpoint=fakes.COMPUTE_ENDPOINT)
        flavor_json = {'flavor': fakes.make_fake_flavor('1', 'vanilla')}
        flavor_extra_json = {'extra_specs': {'name': 'test'}}

        self.register_uris([
            dict(method='GET', uri=flavor_uri, json=flavor_json),
            dict(method='GET', uri=flavor_extra_uri, json=flavor_extra_json),
        ])

        flavor1 = self.cloud.get_flavor_by_id('1', get_extra=True)
        self.assertEqual('1', flavor1['id'])
        self.assertEqual({'name': 'test'}, flavor1.extra_specs)
        flavor2 = self.cloud.get_flavor_by_id('1', get_extra=False)
        self.assertEqual('1', flavor2['id'])
        self.assertEqual({}, flavor2.extra_specs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class UserMessage(resource.Resource):
    resource_key = "message"
    resources_key = "messages"
    base_path = "/messages"

    # capabilities
    allow_fetch = True
    allow_commit = False
    allow_delete = True
    allow_list = True
    allow_head = False

    _query_mapping = resource.QueryParameters(
        "message_id"
    )

    _max_microversion = '2.37'

    #: Properties
    #: The action ID of the user message
    action_id = resource.Body("action_id", type=str)
    #: Indicate when the user message was created
    created_at = resource.Body("created_at", type=str)
    #: The detail ID of the user message
    detail_id = resource.Body("detail_id", type=str)
    #: Indicate when the share message expires
    expires_at = resource.Body("expires_at", type=str)
    #: The message level of the user message
    message_level = resource.Body("message_level", type=str)
    #: The project ID of the user message
    project_id = resource.Body("project_id", type=str)
    #: The request ID of the user message
    request_id = resource.Body("request_id", type=str)
    #: The resource ID of the user message
    resource_id = resource.Body("resource_id", type=str)
    #: The resource type of the user message
    resource_type = resource.Body("resource_type", type=str)
    #: The message for the user message
    user_message = resource.Body("user_message", type=str)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import port_forwarding
from openstack.tests.unit import base


EXAMPLE = {
    'id': 'pf_id',
    'protocol': 'tcp',
    'internal_ip_address': '1.2.3.4',
    'floatingip_id': 'floating-ip-uuid',
    'internal_port': 80,
    'internal_port_id': 'internal-port-uuid',
    'external_port': 8080,
    'description': 'description'
}


class TestFloatingIP(base.TestCase):

    def test_basic(self):
        sot = port_forwarding.PortForwarding()
        self.assertEqual('port_forwarding', sot.resource_key)
        self.assertEqual('port_forwardings', sot.resources_key)
        self.assertEqual(
            '/floatingips/%(floatingip_id)s/port_forwardings',
            sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({'internal_port_id': 'internal_port_id',
                              'external_port': 'external_port',
                              'limit': 'limit',
                              'marker': 'marker',
                              'protocol': 'protocol'},
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = port_forwarding.PortForwarding(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['floatingip_id'], sot.floatingip_id)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)
        self.assertEqual(EXAMPLE['internal_ip_address'],
                         sot.internal_ip_address)
        self.assertEqual(EXAMPLE['internal_port'], sot.internal_port)
        self.assertEqual(EXAMPLE['internal_port_id'], sot.internal_port_id)
        self.assertEqual(EXAMPLE['external_port'], sot.external_port)
        self.assertEqual(EXAMPLE['description'], sot.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_identity
----------------------------------

Functional tests for `shade` identity methods.
"""

import random
import string

from openstack.cloud.exc import OpenStackCloudException
from openstack.tests.functional import base


class TestIdentity(base.KeystoneBaseFunctionalTest):
    def setUp(self):
        super(TestIdentity, self).setUp()
        self.role_prefix = 'test_role' + ''.join(
            random.choice(string.ascii_lowercase) for _ in range(5))
        self.user_prefix = self.getUniqueString('user')
        self.group_prefix = self.getUniqueString('group')

        self.addCleanup(self._cleanup_users)
        if self.identity_version not in ('2', '2.0'):
            self.addCleanup(self._cleanup_groups)
        self.addCleanup(self._cleanup_roles)

    def _cleanup_groups(self):
        exception_list = list()
        for group in self.operator_cloud.list_groups():
            if group['name'].startswith(self.group_prefix):
                try:
                    self.operator_cloud.delete_group(group['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise OpenStackCloudException('\n'.join(exception_list))

    def _cleanup_users(self):
        exception_list = list()
        for user in self.operator_cloud.list_users():
            if user['name'].startswith(self.user_prefix):
                try:
                    self.operator_cloud.delete_user(user['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            raise OpenStackCloudException('\n'.join(exception_list))

    def _cleanup_roles(self):
        exception_list = list()
        for role in self.operator_cloud.list_roles():
            if role['name'].startswith(self.role_prefix):
                try:
                    self.operator_cloud.delete_role(role['name'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            raise OpenStackCloudException('\n'.join(exception_list))

    def _create_user(self, **kwargs):
        domain_id = None
        if self.identity_version not in ('2', '2.0'):
            domain = self.operator_cloud.get_domain('default')
            domain_id = domain['id']
        return self.operator_cloud.create_user(domain_id=domain_id, **kwargs)

    def test_list_roles(self):
        roles = self.operator_cloud.list_roles()
        self.assertIsNotNone(roles)
        self.assertNotEqual([], roles)

    def test_get_role(self):
        role = self.operator_cloud.get_role('admin')
        self.assertIsNotNone(role)
        self.assertIn('id', role)
        self.assertIn('name', role)
        self.assertEqual('admin', role['name'])

    def test_search_roles(self):
        roles = self.operator_cloud.search_roles(filters={'name': 'admin'})
        self.assertIsNotNone(roles)
        self.assertEqual(1, len(roles))
        self.assertEqual('admin', roles[0]['name'])

    def test_create_role(self):
        role_name = self.role_prefix + '_create_role'
        role = self.operator_cloud.create_role(role_name)
        self.assertIsNotNone(role)
        self.assertIn('id', role)
        self.assertIn('name', role)
        self.assertEqual(role_name, role['name'])

    def test_delete_role(self):
        role_name = self.role_prefix + '_delete_role'
        role = self.operator_cloud.create_role(role_name)
        self.assertIsNotNone(role)
        self.assertTrue(self.operator_cloud.delete_role(role_name))

    # TODO(Shrews): Once we can support assigning roles within shade, we
    # need to make this test a little more specific, and add more for testing
    # filtering functionality.
    def test_list_role_assignments(self):
        if self.identity_version in ('2', '2.0'):
            self.skipTest("Identity service does not support role assignments")
        assignments = self.operator_cloud.list_role_assignments()
        self.assertIsInstance(assignments, list)
        self.assertGreater(len(assignments), 0)

    def test_list_role_assignments_v2(self):
        user = self.operator_cloud.get_user('demo')
        project = self.operator_cloud.get_project('demo')
        assignments = self.operator_cloud.list_role_assignments(
            filters={'user': user['id'], 'project': project['id']})
        self.assertIsInstance(assignments, list)
        self.assertGreater(len(assignments), 0)

    def test_grant_revoke_role_user_project(self):
        user_name = self.user_prefix + '_user_project'
        user_email = 'nobody@nowhere.com'
        role_name = self.role_prefix + '_grant_user_project'
        role = self.operator_cloud.create_role(role_name)
        user = self._create_user(name=user_name,
                                 email=user_email,
                                 default_project='demo')
        self.assertTrue(self.operator_cloud.grant_role(
            role_name, user=user['id'], project='demo', wait=True))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'user': user['id'],
            'project': self.operator_cloud.get_project('demo')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(1, len(assignments))
        self.assertTrue(self.operator_cloud.revoke_role(
            role_name, user=user['id'], project='demo', wait=True))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'user': user['id'],
            'project': self.operator_cloud.get_project('demo')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(0, len(assignments))

    def test_grant_revoke_role_group_project(self):
        if self.identity_version in ('2', '2.0'):
            self.skipTest("Identity service does not support group")
        role_name = self.role_prefix + '_grant_group_project'
        role = self.operator_cloud.create_role(role_name)
        group_name = self.group_prefix + '_group_project'
        group = self.operator_cloud.create_group(
            name=group_name,
            description='test group',
            domain='default')
        self.assertTrue(self.operator_cloud.grant_role(
            role_name, group=group['id'], project='demo'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'group': group['id'],
            'project': self.operator_cloud.get_project('demo')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(1, len(assignments))
        self.assertTrue(self.operator_cloud.revoke_role(
            role_name, group=group['id'], project='demo'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'group': group['id'],
            'project': self.operator_cloud.get_project('demo')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(0, len(assignments))

    def test_grant_revoke_role_user_domain(self):
        if self.identity_version in ('2', '2.0'):
            self.skipTest("Identity service does not support domain")
        role_name = self.role_prefix + '_grant_user_domain'
        role = self.operator_cloud.create_role(role_name)
        user_name = self.user_prefix + '_user_domain'
        user_email = 'nobody@nowhere.com'
        user = self._create_user(name=user_name,
                                 email=user_email,
                                 default_project='demo')
        self.assertTrue(self.operator_cloud.grant_role(
            role_name, user=user['id'], domain='default'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'user': user['id'],
            'domain': self.operator_cloud.get_domain('default')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(1, len(assignments))
        self.assertTrue(self.operator_cloud.revoke_role(
            role_name, user=user['id'], domain='default'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'user': user['id'],
            'domain': self.operator_cloud.get_domain('default')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(0, len(assignments))

    def test_grant_revoke_role_group_domain(self):
        if self.identity_version in ('2', '2.0'):
            self.skipTest("Identity service does not support domain or group")
        role_name = self.role_prefix + '_grant_group_domain'
        role = self.operator_cloud.create_role(role_name)
        group_name = self.group_prefix + '_group_domain'
        group = self.operator_cloud.create_group(
            name=group_name,
            description='test group',
            domain='default')
        self.assertTrue(self.operator_cloud.grant_role(
            role_name, group=group['id'], domain='default'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'group': group['id'],
            'domain': self.operator_cloud.get_domain('default')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(1, len(assignments))
        self.assertTrue(self.operator_cloud.revoke_role(
            role_name, group=group['id'], domain='default'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'group': group['id'],
            'domain': self.operator_cloud.get_domain('default')['id']
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(0, len(assignments))

    def test_grant_revoke_role_user_system(self):
        role_name = self.role_prefix + '_grant_user_system'
        role = self.operator_cloud.create_role(role_name)
        user_name = self.user_prefix + '_user_system'
        user_email = 'nobody@nowhere.com'
        user = self._create_user(name=user_name,
                                 email=user_email,
                                 default_project='demo')
        self.assertTrue(self.operator_cloud.grant_role(
            role_name, user=user['id'], system='all'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'user': user['id'],
            'system': 'all'
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(1, len(assignments))
        self.assertTrue(self.operator_cloud.revoke_role(
            role_name, user=user['id'], system='all'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'user': user['id'],
            'system': 'all'
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(0, len(assignments))

    def test_grant_revoke_role_group_system(self):
        if self.identity_version in ('2', '2.0'):
            self.skipTest("Identity service does not support system or group")
        role_name = self.role_prefix + '_grant_group_system'
        role = self.operator_cloud.create_role(role_name)
        group_name = self.group_prefix + '_group_system'
        group = self.operator_cloud.create_group(
            name=group_name,
            description='test group')
        self.assertTrue(self.operator_cloud.grant_role(
            role_name, group=group['id'], system='all'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'group': group['id'],
            'system': 'all'
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(1, len(assignments))
        self.assertTrue(self.operator_cloud.revoke_role(
            role_name, group=group['id'], system='all'))
        assignments = self.operator_cloud.list_role_assignments({
            'role': role['id'],
            'group': group['id'],
            'system': 'all'
        })
        self.assertIsInstance(assignments, list)
        self.assertEqual(0, len(assignments))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import availability_zone as az
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "id": IDENTIFIER,
    "zoneState": {
        "available": True
    },
    "zoneName": "zone1"
}


class TestAvailabilityZone(base.TestCase):

    def test_basic(self):
        sot = az.AvailabilityZone()
        self.assertEqual('availabilityZoneInfo', sot.resources_key)
        self.assertEqual('/os-availability-zone', sot.base_path)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = az.AvailabilityZone(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['zoneState'], sot.state)
        self.assertEqual(EXAMPLE['zoneName'], sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import _base
from openstack import resource


class Recordset(_base.Resource):
    """DNS Recordset Resource"""
    resources_key = 'recordsets'
    base_path = '/zones/%(zone_id)s/recordsets'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'name', 'type', 'ttl', 'data', 'status', 'description',
        'limit', 'marker')

    #: Properties
    #: current action in progress on the resource
    action = resource.Body('action')
    #: Timestamp when the zone was created
    created_at = resource.Body('create_at')
    #: Recordset description
    description = resource.Body('description')
    #: Links contains a `self` pertaining to this zone or a `next` pertaining
    #: to next page
    links = resource.Body('links', type=dict)
    #: DNS Name of the recordset
    name = resource.Body('name')
    #: ID of the project which the recordset belongs to
    project_id = resource.Body('project_id')
    #: DNS record value list
    records = resource.Body('records', type=list)
    #: Recordset status
    #: Valid values include: `PENDING_CREATE`, `ACTIVE`,`PENDING_DELETE`,
    #: `ERROR`
    status = resource.Body('status')
    #: Time to live, default 300, available value 300-2147483647 (seconds)
    ttl = resource.Body('ttl', type=int)
    #: DNS type of the recordset
    #: Valid values include `A`, `AAAA`, `MX`, `CNAME`, `TXT`, `NS`,
    #: `SSHFP`, `SPF`, `SRV`, `PTR`
    type = resource.Body('type')
    #: Timestamp when the zone was last updated
    updated_at = resource.Body('updated_at')
    #: The id of the Zone which this recordset belongs to
    zone_id = resource.URI('zone_id')
    #: The name of the Zone which this recordset belongs to
    zone_name = resource.Body('zone_name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v3 import role_system_user_assignment
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'name': '2',
    'user_id': '4'
}


class TestRoleSystemUserAssignment(base.TestCase):

    def test_basic(self):
        sot = role_system_user_assignment.RoleSystemUserAssignment()
        self.assertEqual('role', sot.resource_key)
        self.assertEqual('roles', sot.resources_key)
        self.assertEqual('/system/users/%(user_id)s/roles',
                         sot.base_path)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = \
            role_system_user_assignment.RoleSystemUserAssignment(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import service_profile
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE_WITH_OPTIONAL = {
    'description': 'test flavor profile',
    'driver': 'neutron_lbaas.drivers.octavia.driver.OctaviaDriver',
    'enabled': True,
    'metainfo': {'foo': 'bar'},
    'project_id': '5',
}

EXAMPLE = {
    'driver': 'neutron_lbaas.drivers.octavia.driver.OctaviaDriver',
}


class TestServiceProfile(base.TestCase):
    def test_basic(self):
        service_profiles = service_profile.ServiceProfile()
        self.assertEqual('service_profile', service_profiles.resource_key)
        self.assertEqual('service_profiles', service_profiles.resources_key)
        self.assertEqual('/service_profiles', service_profiles.base_path)
        self.assertTrue(service_profiles.allow_create)
        self.assertTrue(service_profiles.allow_fetch)
        self.assertTrue(service_profiles.allow_commit)
        self.assertTrue(service_profiles.allow_delete)
        self.assertTrue(service_profiles.allow_list)

    def test_make_it(self):
        service_profiles = service_profile.ServiceProfile(**EXAMPLE)
        self.assertEqual(EXAMPLE['driver'], service_profiles.driver)

    def test_make_it_with_optional(self):
        service_profiles = service_profile.ServiceProfile(
            **EXAMPLE_WITH_OPTIONAL)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['description'],
                         service_profiles.description)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['driver'],
                         service_profiles.driver)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['enabled'],
                         service_profiles.is_enabled)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['metainfo'],
                         service_profiles.meta_info)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['project_id'],
                         service_profiles.project_id)
